@@ -25,7 +25,7 @@
 
 //--------------------------------------------------------------------------
 
-std::mutex g_clangMutex;
+static std::mutex g_clangMutex;
 
 ClangParser *ClangParser::instance()
 {
@@ -39,6 +39,8 @@ ClangParser *ClangParser::s_instance = 0;
 //--------------------------------------------------------------------------
 #if USE_LIBCLANG
 
+static std::mutex g_docCrossReferenceMutex;
+
 enum class DetectedLang { Cpp, ObjC, ObjCpp };
 
 static QCString detab(const QCString &s)
@@ -121,7 +123,7 @@ class ClangTUParser::Private
       : parser(p), fileDef(fd) {}
     const ClangParser &parser;
     const FileDef *fileDef;
-    CXIndex index;
+    CXIndex index = 0;
     uint curToken = 0;
     DetectedLang detectedLang = DetectedLang::Cpp;
     uint numFiles = 0;
@@ -129,7 +131,7 @@ class ClangTUParser::Private
     std::vector<CXUnsavedFile> ufs;
     std::vector<CXCursor> cursors;
     std::unordered_map<std::string,uint> fileMapping;
-    CXTranslationUnit tu;
+    CXTranslationUnit tu = 0;
     CXToken *tokens = 0;
     uint numTokens = 0;
     StringVector filesInSameTU;
@@ -145,6 +147,7 @@ class ClangTUParser::Private
 ClangTUParser::ClangTUParser(const ClangParser &parser,const FileDef *fd)
   : p(std::make_unique<Private>(parser,fd))
 {
+  //printf("ClangTUParser::ClangTUParser() this=%p\n",this);
 }
 
 StringVector ClangTUParser::filesInSameTU() const
@@ -154,6 +157,7 @@ StringVector ClangTUParser::filesInSameTU() const
 
 void ClangTUParser::parse()
 {
+  //printf("ClangTUParser::parse() this=%p\n",this);
   QCString fileName = p->fileDef->absFilePath();
   p->fileDef->getAllIncludeFilesRecursively(p->filesInSameTU);
   //printf("ClangTUParser::ClangTUParser(fileName=%s,#filesInSameTU=%d)\n",
@@ -164,8 +168,12 @@ void ClangTUParser::parse()
   const StringVector &clangOptions = Config_getList(CLANG_OPTIONS);
   if (!clangAssistedParsing) return;
   //printf("ClangParser::start(%s)\n",fileName);
+  assert(p->index==0);
+  assert(p->tokens==0);
+  assert(p->numTokens==0);
   p->index    = clang_createIndex(0, 0);
   p->curToken = 0;
+  p->cursors.clear();
   int argc=0;
   size_t clang_option_len = 0;
   std::vector<clang::tooling::CompileCommand> command;
@@ -289,7 +297,7 @@ void ClangTUParser::parse()
   // free arguments
   for (i=0;i<argc;++i)
   {
-    free(argv[i]);
+    delete[](argv[i]);
   }
   free(argv);
 
@@ -309,16 +317,13 @@ void ClangTUParser::parse()
   }
   else
   {
-    p->tokens    = 0;
-    p->numTokens = 0;
-    p->cursors.clear();
     err("clang: Failed to parse translation unit %s\n",qPrint(fileName));
   }
 }
 
 ClangTUParser::~ClangTUParser()
 {
-  //printf("ClangTUParser::~ClangTUParser() tu=%p\n",p->tu);
+  //printf("ClangTUParser::~ClangTUParser() this=%p\n",this);
   static bool clangAssistedParsing = Config_getBool(CLANG_ASSISTED_PARSING);
   if (!clangAssistedParsing) return;
   if (p->tu)
@@ -333,7 +338,7 @@ ClangTUParser::~ClangTUParser()
   }
   for (uint i=0;i<p->numFiles;i++)
   {
-    free((void *)p->ufs[i].Filename);
+    delete[] p->ufs[i].Filename;
   }
   p->ufs.clear();
   p->sources.clear();
@@ -343,7 +348,7 @@ ClangTUParser::~ClangTUParser()
 
 void ClangTUParser::switchToFile(FileDef *fd)
 {
-  //printf("ClangTUParser::switchToFile(%s)\n",qPrint(fd->absFilePath()));
+  //printf("ClangTUParser::switchToFile(%s) this=%p\n",qPrint(fd->absFilePath()),this);
   if (p->tu)
   {
     p->cursors.clear();
@@ -569,7 +574,7 @@ void ClangTUParser::writeMultiLineCodeLink(CodeOutputInterface &ol,
                   const char *text)
 {
   static bool sourceTooltips = Config_getBool(SOURCE_TOOLTIPS);
-  TooltipManager::instance()->addTooltip(d);
+  TooltipManager::instance().addTooltip(ol,d);
   QCString ref  = d->getReference();
   QCString file = d->getOutputFileBase();
   QCString anchor = d->anchor();
@@ -692,7 +697,8 @@ void ClangTUParser::linkIdentifier(CodeOutputInterface &ol,FileDef *fd,
         p->currentMemberDef && d->definitionType()==Definition::TypeMember &&
         (p->currentMemberDef!=d || p->currentLine<line)) // avoid self-reference
     {
-      addDocCrossReference(p->currentMemberDef,dynamic_cast<MemberDef *>(d));
+      std::lock_guard<std::mutex> lock(g_docCrossReferenceMutex);
+      addDocCrossReference(toMemberDefMutable(p->currentMemberDef),toMemberDefMutable(d));
     }
     writeMultiLineCodeLink(ol,fd,line,column,d,text);
   }
@@ -855,7 +861,7 @@ class ClangParser::Private
       QCString clangCompileDatabase = Config_getString(CLANG_DATABASE_PATH);
       // load a clang compilation database (https://clang.llvm.org/docs/JSONCompilationDatabase.html)
       db = clang::tooling::CompilationDatabase::loadFromDirectory(clangCompileDatabase.data(), error);
-      if (clangCompileDatabase!="0" && db==nullptr)
+      if (!clangCompileDatabase.isEmpty() && clangCompileDatabase!="0" && db==nullptr)
       {
           // user specified a path, but DB file was not found
           err("%s using clang compilation database path of: \"%s\"\n", error.c_str(),
@@ -881,6 +887,7 @@ ClangParser::~ClangParser()
 
 std::unique_ptr<ClangTUParser> ClangParser::createTUParser(const FileDef *fd) const
 {
+  //printf("ClangParser::createTUParser()\n");
   return std::make_unique<ClangTUParser>(*this,fd);
 }
 
