@@ -1,12 +1,12 @@
 /******************************************************************************
  *
- * $Id: classlist.h,v 1.8 2001/03/19 19:27:39 root Exp $
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ *
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -21,6 +21,7 @@
 #include <qlist.h>
 #include <qdict.h>
 
+#include "linkedmap.h"
 #include "classdef.h"
 #include "sortdict.h"
 
@@ -28,12 +29,13 @@ class Definition;
 
 /** A list of ClassDef objects. */
 class ClassList : public QList<ClassDef>
-{ 
+{
   public:
     ClassList();
    ~ClassList();
-   
-   int compareItems(GCI item1,GCI item2);
+
+  private:
+    int compareValues(const ClassDef *item1,const ClassDef *item2) const;
 };
 
 /** An iterator for ClassDef objects in a ClassList. */
@@ -43,27 +45,6 @@ class ClassListIterator : public QListIterator<ClassDef>
     ClassListIterator(const ClassList &list);
 };
 
-/** An unsorted dictionary of ClassDef objects. */
-class ClassDict : public QDict<ClassDef>
-{
-  public:
-    ClassDict(int size) : QDict<ClassDef>(size) {}
-   ~ClassDict() {}
-};
-
-/** A sorted dictionary of ClassDef objects. */
-class ClassSDict : public SDict<ClassDef>
-{
-  public:
-    ClassSDict(int size=17) : SDict<ClassDef>(size) {}
-   ~ClassSDict() {}
-   int compareItems(GCI item1,GCI item2);
-   void writeDeclaration(OutputList &ol,const ClassDef::CompoundType *filter=0,
-                         const char *header=0,bool localNames=FALSE);
-   void writeDocumentation(OutputList &ol,Definition *container=0);
-   bool declVisible(const ClassDef::CompoundType *filter=0) const;
-};
-
 class GenericsCollection : public QIntDict<ClassDef>
 {
   public:
@@ -71,7 +52,7 @@ class GenericsCollection : public QIntDict<ClassDef>
    ~GenericsCollection() {}
 };
 
-class GenericsSDict 
+class GenericsSDict
 {
   public:
    GenericsSDict() : m_dict(17) { m_dict.setAutoDelete(TRUE); }
@@ -82,4 +63,18 @@ class GenericsSDict
    SDict<GenericsCollection> m_dict;
 };
 
+class ClassLinkedMap : public LinkedMap<ClassDef>
+{
+};
+
+class ClassLinkedRefMap : public LinkedRefMap<const ClassDef>
+{
+  public:
+    bool declVisible(const ClassDef::CompoundType *filter=0) const;
+    void writeDeclaration(OutputList &ol,const ClassDef::CompoundType *filter,const char *header,bool localNames) const;
+    void writeDocumentation(OutputList &ol,const Definition * container=0) const;
+};
+
+
+
 #endif
