@@ -20,21 +20,20 @@
 #include "template.h"
 #include <qlist.h>
 #include <stdio.h>
+#include "dirdef.h"
+#include "classdef.h"
 
 class Definition;
-class ClassDef;
-class ClassSDict;
-class BaseClassList;
 class PageDef;
 class GroupDef;
 class NamespaceDef;
-class BaseClassList;
-class NamespaceSDict;
+class NamespaceLinkedMap;
+class NamespaceLinkedRefMap;
 class FileDef;
 class FileList;
 class FileNameLinkedMap;
+class ClassLinkedMap;
 class DirSDict;
-class DirList;
 class DirDef;
 class PageSDict;
 class GroupSDict;
@@ -46,8 +45,8 @@ class MemberSDict;
 class MemberDef;
 struct Argument;
 class ArgumentList;
-class MemberNameInfoSDict;
-struct MemberInfo;
+class MemberNameInfoLinkedMap;
+class MemberInfo;
 class MemberGroup;
 class MemberGroupSDict;
 class MemberGroupList;
@@ -348,7 +347,7 @@ class PageContext : public RefCountedContext, public TemplateStructIntf
 class MemberContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static MemberContext *alloc(MemberDef *md) { return new MemberContext(md); }
+    static MemberContext *alloc(const MemberDef *md) { return new MemberContext(md); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -356,7 +355,7 @@ class MemberContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    MemberContext(MemberDef *);
+    MemberContext(const MemberDef *);
    ~MemberContext();
     class Private;
     Private *p;
@@ -455,7 +454,7 @@ class ClassInheritanceNodeContext : public RefCountedContext, public TemplateStr
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
-    void addChildren(const BaseClassList *bcl,bool hideSuper);
+    void addChildren(const BaseClassList &bcl,bool hideSuper);
 
   private:
     ClassInheritanceNodeContext(const ClassDef *);
@@ -510,8 +509,9 @@ class NestingNodeContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
     static NestingNodeContext *alloc(const NestingNodeContext *parent,const Definition *def,
-                                     int index,int level,bool addClasses,bool inherit,bool hideSuper)
-    { return new NestingNodeContext(parent,def,index,level,addClasses,inherit,hideSuper); }
+                                     int index,int level,bool addClasses,bool inherit,bool hideSuper,
+                                     ClassDefSet &visitedClasses)
+    { return new NestingNodeContext(parent,def,index,level,addClasses,inherit,hideSuper,visitedClasses); }
 
     QCString id() const;
 
@@ -522,7 +522,8 @@ class NestingNodeContext : public RefCountedContext, public TemplateStructIntf
 
   private:
     NestingNodeContext(const NestingNodeContext *parent,
-                       const Definition *,int index,int level,bool addClasses,bool inherit,bool hideSuper);
+                       const Definition *,int index,int level,bool addClasses,bool inherit,bool hideSuper,
+                       ClassDefSet &visitedClasses);
    ~NestingNodeContext();
     class Private;
     Private *p;
@@ -533,6 +534,7 @@ class NestingNodeContext : public RefCountedContext, public TemplateStructIntf
 class NestingContext : public RefCountedContext, public TemplateListIntf
 {
   public:
+
     static NestingContext *alloc(const NestingNodeContext *parent,int level)
     { return new NestingContext(parent,level); }
 
@@ -543,17 +545,19 @@ class NestingContext : public RefCountedContext, public TemplateListIntf
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
-    void addNamespaces(const NamespaceSDict &nsDict,bool rootOnly,bool addClasses);
-    void addClasses(const ClassSDict &clDict,bool rootOnly);
-    void addDirs(const DirSDict &);
-    void addDirs(const DirList &);
-    void addFiles(const FileNameLinkedMap &);
-    void addFiles(const FileList &);
-    void addPages(const PageSDict &pages,bool rootOnly);
-    void addModules(const GroupSDict &modules);
-    void addModules(const GroupList &modules);
-    void addClassHierarchy(const ClassSDict &clDict,bool rootOnly);
-    void addDerivedClasses(const BaseClassList *bcl,bool hideSuper);
+    void addNamespaces(const NamespaceLinkedMap &nsLinkedMap,bool rootOnly,bool addClasses,ClassDefSet &visitedClasses);
+    void addNamespaces(const NamespaceLinkedRefMap &nsLinkedMap,bool rootOnly,bool addClasses,ClassDefSet &visitedClasses);
+    void addClasses(const ClassLinkedRefMap &clLinkedMap,bool rootOnly,ClassDefSet &visitedClasses);
+    void addClasses(const ClassLinkedMap &clLinkedMap,bool rootOnly,ClassDefSet &visitedClasses);
+    void addDirs(const DirSDict &,ClassDefSet &visitedClasses);
+    void addDirs(const DirList &,ClassDefSet &visitedClasses);
+    void addFiles(const FileNameLinkedMap &,ClassDefSet &visitedClasses);
+    void addFiles(const FileList &,ClassDefSet &visitedClasses);
+    void addPages(const PageSDict &pages,bool rootOnly,ClassDefSet &visitedClasses);
+    void addModules(const GroupSDict &modules,ClassDefSet &visitedClasses);
+    void addModules(const GroupList &modules,ClassDefSet &visitedClasses);
+    void addClassHierarchy(const ClassLinkedMap &clLinkedMap,ClassDefSet &visitedClasses);
+    void addDerivedClasses(const BaseClassList &bcl,bool hideSuper,ClassDefSet &visitedClasses);
 
   private:
     NestingContext(const NestingNodeContext *parent,int level);
@@ -929,7 +933,7 @@ class InheritanceNodeContext : public RefCountedContext, public TemplateStructIn
 class InheritanceListContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static InheritanceListContext *alloc(const BaseClassList *list,bool baseClasses)
+    static InheritanceListContext *alloc(const BaseClassList &list,bool baseClasses)
     { return new InheritanceListContext(list,baseClasses); }
 
     // TemplateListIntf
@@ -940,7 +944,7 @@ class InheritanceListContext : public RefCountedContext, public TemplateListIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    InheritanceListContext(const BaseClassList *list,bool baseClasses);
+    InheritanceListContext(const BaseClassList &list,bool baseClasses);
    ~InheritanceListContext();
     class Private;
     Private *p;
@@ -957,6 +961,8 @@ class MemberListContext : public RefCountedContext, public TemplateListIntf
     { return new MemberListContext(ml); }
     static MemberListContext *alloc(MemberSDict *ml,bool doSort)
     { return new MemberListContext(ml,doSort); }
+    static MemberListContext *alloc(std::vector<const MemberDef *> &&ml)
+    { return new MemberListContext(std::move(ml)); }
 
     // TemplateListIntf
     virtual uint count() const;
@@ -969,6 +975,7 @@ class MemberListContext : public RefCountedContext, public TemplateListIntf
     MemberListContext();
     MemberListContext(const MemberList *ml);
     MemberListContext(MemberSDict *ml,bool doSort);
+    MemberListContext(std::vector<const MemberDef *> &&ml);
    ~MemberListContext();
     class Private;
     Private *p;
@@ -1113,9 +1120,7 @@ class InheritedMemberInfoListContext : public RefCountedContext, public Template
 class AllMembersListContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static AllMembersListContext *alloc()
-    { return new AllMembersListContext; }
-    static AllMembersListContext *alloc(const MemberNameInfoSDict *ml)
+    static AllMembersListContext *alloc(const MemberNameInfoLinkedMap &ml)
     { return new AllMembersListContext(ml); }
 
     // TemplateListIntf
@@ -1126,8 +1131,7 @@ class AllMembersListContext : public RefCountedContext, public TemplateListIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    AllMembersListContext();
-    AllMembersListContext(const MemberNameInfoSDict *ml);
+    AllMembersListContext(const MemberNameInfoLinkedMap &ml);
    ~AllMembersListContext();
     class Private;
     Private *p;
