@@ -15,10 +15,11 @@
 #include "docparser.h"
 #include "definitionimpl.h"
 #include "filedef.h"
+#include <algorithm>
 
 //----------------------------------------------------------------------
 
-class DirDefImpl : public DefinitionImpl, public DirDef
+class DirDefImpl : public DefinitionMixin<DirDef>
 {
   public:
     DirDefImpl(const char *path);
@@ -35,7 +36,7 @@ class DirDefImpl : public DefinitionImpl, public DirDef
     virtual FileList *   getFiles() const        { return m_fileList; }
     virtual void addFile(FileDef *fd);
     virtual const DirList &subDirs() const { return m_subdirs; }
-    virtual bool isCluster() const { return m_subdirs.count()>0; }
+    virtual bool isCluster() const { return m_subdirs.size()>0; }
     virtual int level() const { return m_level; }
     virtual DirDef *parent() const { return m_parent; }
     virtual int dirCount() const { return m_dirCount; }
@@ -68,7 +69,7 @@ class DirDefImpl : public DefinitionImpl, public DirDef
     void endMemberDeclarations(OutputList &ol);
 
     static DirDef *createNewDir(const char *path);
-    static bool matchPath(const QCString &path,QStrList &l);
+    static bool matchPath(const QCString &path,const StringVector &l);
 
     DirList m_subdirs;
     QCString m_dispName;
@@ -92,7 +93,7 @@ DirDef *createDirDef(const char *path)
 
 static int g_dirCount=0;
 
-DirDefImpl::DirDefImpl(const char *path) : DefinitionImpl(path,1,1,path)
+DirDefImpl::DirDefImpl(const char *path) : DefinitionMixin(path,1,1,path)
 {
   bool fullPathNames = Config_getBool(FULL_PATH_NAMES);
   // get display name (stripping the paths mentioned in STRIP_FROM_PATH)
@@ -141,7 +142,7 @@ bool DirDefImpl::isLinkable() const
 
 void DirDefImpl::addSubDir(DirDef *subdir)
 {
-  m_subdirs.append(subdir);
+  m_subdirs.push_back(subdir);
   subdir->setOuterScope(this);
   subdir->setParent(this);
 }
@@ -159,7 +160,7 @@ void DirDefImpl::addFile(FileDef *fd)
 
 void DirDefImpl::sort()
 {
-  m_subdirs.sort();
+  std::sort(m_subdirs.begin(), m_subdirs.end(), compareDirDefs);
   m_fileList->sort();
 }
 
@@ -222,7 +223,8 @@ void DirDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
     // repeat brief description
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF))
     {
-      ol.generateDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
+      ol.generateDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE,
+                     0,FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
     }
     // separator between brief and details
     if (!briefDescription().isEmpty() && Config_getBool(REPEAT_BRIEF) &&
@@ -242,7 +244,8 @@ void DirDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title)
     // write documentation
     if (!documentation().isEmpty())
     {
-      ol.generateDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE);
+      ol.generateDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE,
+                     0,FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
     }
   }
 }
@@ -252,7 +255,8 @@ void DirDefImpl::writeBriefDescription(OutputList &ol)
   if (hasBriefDescription())
   {
     DocRoot *rootNode = validatingParseDoc(
-         briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
+         briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE,
+         0,FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
     if (rootNode && !rootNode->isEmpty())
     {
       ol.startParagraph();
@@ -307,9 +311,7 @@ void DirDefImpl::writeDirectoryGraph(OutputList &ol)
 void DirDefImpl::writeSubDirList(OutputList &ol)
 {
   int numSubdirs = 0;
-  QListIterator<DirDef> it(m_subdirs);
-  DirDef *dd;
-  for (it.toFirst();(dd=it.current());++it)
+  for(const auto dd : m_subdirs)
   {
     if (dd->hasDocumentation() || dd->getFiles()->count()>0)
     {
@@ -324,7 +326,7 @@ void DirDefImpl::writeSubDirList(OutputList &ol)
     ol.parseText(theTranslator->trDir(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
-    for (it.toFirst();(dd=it.current());++it)
+    for(const auto dd : m_subdirs)
     {
       if (dd->hasDocumentation() || dd->getFiles()->count()==0)
       {
@@ -342,7 +344,8 @@ void DirDefImpl::writeSubDirList(OutputList &ol)
               FALSE, // isExample
               0,     // exampleName
               TRUE,  // single line
-              TRUE   // link from index
+              TRUE,  // link from index
+              Config_getBool(MARKDOWN_SUPPORT)
               );
           ol.endMemberDescription();
         }
@@ -413,7 +416,8 @@ void DirDefImpl::writeFileList(OutputList &ol)
               FALSE, // isExample
               0,     // exampleName
               TRUE,  // single line
-              TRUE   // link from index
+              TRUE,  // link from index
+              Config_getBool(MARKDOWN_SUPPORT)
               );
           ol.endMemberDescription();
         }
@@ -460,11 +464,9 @@ void DirDefImpl::writeTagFile(FTextStream &tagFile)
     {
       case LayoutDocEntry::DirSubDirs:
         {
-          if (m_subdirs.count()>0)
+          if (m_subdirs.size()>0)
           {
-            DirDef *dd;
-            QListIterator<DirDef> it(m_subdirs);
-            for (;(dd=it.current());++it)
+            for(const auto dd : m_subdirs)
             {
               tagFile << "    <dir>" << convertToXML(dd->displayName()) << "</dir>" << endl;
             }
@@ -753,7 +755,7 @@ int FilePairDict::compareValues(const FilePair *left,const FilePair *right) cons
 
 //----------------------------------------------------------------------
 
-UsedDir::UsedDir(DirDef *dir,bool inherited) :
+UsedDir::UsedDir(const DirDef *dir,bool inherited) :
    m_dir(dir), m_filePairs(7), m_inherited(inherited)
 {
   m_filePairs.setAutoDelete(TRUE);
@@ -795,17 +797,15 @@ DirDef *DirDefImpl::createNewDir(const char *path)
   return dir;
 }
 
-bool DirDefImpl::matchPath(const QCString &path,QStrList &l)
+bool DirDefImpl::matchPath(const QCString &path,const StringVector &l)
 {
-  const char *s=l.first();
-  while (s)
+  for (const auto &s : l)
   {
-    QCString prefix = s;
-    if (qstricmp(prefix.left(path.length()),path)==0) // case insensitive compare
+    std::string prefix = s.substr(0,path.length());
+    if (qstricmp(prefix.c_str(),path)==0) // case insensitive compare
     {
       return TRUE;
     }
-    s = l.next();
   }
   return FALSE;
 }
@@ -1096,3 +1096,36 @@ void generateDirDocs(OutputList &ol)
   }
 }
 
+bool compareDirDefs(const DirDef *item1, const DirDef *item2)
+{
+  return qstricmp(item1->shortName(),item2->shortName()) < 0;
+}
+
+// --- Cast functions
+
+DirDef *toDirDef(Definition *d)
+{
+  if (d==0) return 0;
+  if (d && typeid(*d)==typeid(DirDefImpl))
+  {
+    return static_cast<DirDef*>(d);
+  }
+  else
+  {
+    return 0;
+  }
+}
+
+const DirDef *toDirDef(const Definition *d)
+{
+  if (d==0) return 0;
+  if (d && typeid(*d)==typeid(DirDefImpl))
+  {
+    return static_cast<const DirDef*>(d);
+  }
+  else
+  {
+    return 0;
+  }
+}
+
