@@ -1,12 +1,10 @@
 /******************************************************************************
  *
- * 
- *
- * Copyright (C) 1997-2015 by Dimitri van Heesch.
+ * Copyright (C) 1997-2020 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -21,10 +19,11 @@
 #include "sortdict.h"
 #include "definition.h"
 
-#include <qlist.h>
+#include <vector>
+#include <qglobal.h>
+#include <qcstring.h>
 
 class FileList;
-class ClassSDict;
 class QStrList;
 class FileDef;
 class OutputList;
@@ -34,14 +33,12 @@ class FTextStream;
 class DirDef;
 
 /** A list of directories. */
-class DirList : public QList<DirDef>
-{
-  public:
-   int compareValues(const DirDef *item1,const DirDef *item2) const;
-};
+typedef std::vector<DirDef*> DirList;
+
+bool compareDirDefs(const DirDef *item1, const DirDef *item2);
 
 /** A model of a directory symbol. */
-class DirDef : virtual public Definition
+class DirDef : public DefinitionMutable, public Definition
 {
   public:
     virtual ~DirDef() {}
@@ -81,8 +78,16 @@ class DirDef : virtual public Definition
     virtual void computeDependencies() = 0;
 };
 
+// --- Cast functions
+
+DirDef            *toDirDef(Definition *d);
+const DirDef      *toDirDef(const Definition *d);
+
+// ------------------
+
+
 /** Class representing a pair of FileDef objects */
-class FilePair 
+class FilePair
 {
   public:
     FilePair(FileDef *src,FileDef *dst) : m_src(src), m_dst(dst) {}
@@ -106,7 +111,7 @@ class FilePairDict : public SDict<FilePair>
 class UsedDir
 {
   public:
-    UsedDir(DirDef *dir,bool inherited);
+    UsedDir(const DirDef *dir,bool inherited);
     virtual ~UsedDir();
     void addFileDep(FileDef *srcFd,FileDef *dstFd);
     FilePair *findFilePair(const char *name);
@@ -116,7 +121,7 @@ class UsedDir
     void sort();
 
   private:
-    DirDef *m_dir;
+    const DirDef *m_dir;
     FilePairDict m_filePairs;
     bool m_inherited;
 };
@@ -125,7 +130,7 @@ class UsedDir
 class DirRelation
 {
   public:
-    DirRelation(const QCString &name,const DirDef *src,UsedDir *dst) 
+    DirRelation(const QCString &name,const DirDef *src,UsedDir *dst)
       : m_name(name), m_src(src), m_dst(dst) {}
     const DirDef  *source() const      { return m_src; }
     UsedDir *destination() const { return m_dst; }
@@ -138,11 +143,6 @@ class DirRelation
     UsedDir *m_dst;
 };
 
-inline int DirList::compareValues(const DirDef *item1,const DirDef *item2) const
-{
-  return qstricmp(item1->shortName(),item2->shortName());
-}
-
 /** A sorted dictionary of DirDef objects. */
 class DirSDict : public SDict<DirDef>
 {
