@@ -17,16 +17,23 @@
 #define DOXYGEN_H
 
 #include <qdatetime.h>
-#include <qcache.h>
 #include <qstrlist.h>
 #include <qdict.h>
 #include <qintdict.h>
 
+#include "containers.h"
 #include "ftextstream.h"
 #include "sortdict.h"
 #include "membergroup.h"
 #include "dirdef.h"
 #include "memberlist.h"
+#include "define.h"
+#include "cache.h"
+#include "symbolmap.h"
+
+#define THREAD_LOCAL thread_local
+#define AtomicInt    std::atomic_int
+#define AtomicBool   std::atomic_bool
 
 class RefList;
 class PageSList;
@@ -34,7 +41,6 @@ class PageSDict;
 class PageDef;
 class SearchIndexIntf;
 class ParserManager;
-class Store;
 class QFileInfo;
 class BufStr;
 class CiteDict;
@@ -43,13 +49,12 @@ class GroupDef;
 class GroupSDict;
 class FileDef;
 class ClassDef;
-class ClassSDict;
+class ClassLinkedMap;
 class GenericsSDict;
 class MemberNameLinkedMap;
 class FileNameLinkedMap;
-class NamespaceSDict;
+class NamespaceLinkedMap;
 class NamespaceDef;
-class DefinitionIntf;
 class DirSDict;
 class DirRelation;
 class IndexList;
@@ -58,11 +63,10 @@ class FormulaDict;
 class FormulaNameDict;
 class Preprocessor;
 struct MemberGroupInfo;
+class NamespaceDefMutable;
 
 typedef QList<QCString>    StringList;
 typedef QListIterator<QCString>    StringListIterator;
-//typedef QDict<FileDef>     FileDict;
-//typedef QDict<GroupDef>    GroupDict;
 
 class StringDict : public QDict<QCString>
 {
@@ -73,11 +77,11 @@ class StringDict : public QDict<QCString>
 
 struct LookupInfo
 {
-  LookupInfo() : classDef(0), typeDef(0) {}
+  LookupInfo() = default;
   LookupInfo(const ClassDef *cd,const MemberDef *td,QCString ts,QCString rt)
     : classDef(cd), typeDef(td), templSpec(ts),resolvedType(rt) {}
-  const ClassDef  *classDef;
-  const MemberDef *typeDef;
+  const ClassDef  *classDef = 0;
+  const MemberDef *typeDef = 0;
   QCString   templSpec;
   QCString   resolvedType;
 };
@@ -91,15 +95,15 @@ extern QCString g_spaces;
 class Doxygen
 {
   public:
-    static ClassSDict               *classSDict;
-    static ClassSDict               *hiddenClasses;
+    static ClassLinkedMap           *classLinkedMap;
+    static ClassLinkedMap           *hiddenClassLinkedMap;
     static PageSDict                *exampleSDict;
     static PageSDict                *pageSDict;
     static PageDef                  *mainPage;
     static bool                      insideMainPage;
     static FileNameLinkedMap        *includeNameLinkedMap;
     static FileNameLinkedMap        *exampleNameLinkedMap;
-    static QDict<void>               inputPaths;
+    static StringSet                 inputPaths;
     static FileNameLinkedMap        *inputNameLinkedMap;
     static FileNameLinkedMap        *imageNameLinkedMap;
     static FileNameLinkedMap        *dotFileNameLinkedMap;
@@ -107,29 +111,26 @@ class Doxygen
     static FileNameLinkedMap        *diaFileNameLinkedMap;
     static MemberNameLinkedMap      *memberNameLinkedMap;
     static MemberNameLinkedMap      *functionNameLinkedMap;
-    static QStrList                  tagfileList;
-    static StringDict                namespaceAliasDict;
+    static StringUnorderedMap        namespaceAliasMap;
     static GroupSDict               *groupSDict;
-    static NamespaceSDict           *namespaceSDict;
+    static NamespaceLinkedMap       *namespaceLinkedMap;
     static StringDict                tagDestinationDict;
     static StringDict                aliasDict;
     static QIntDict<MemberGroupInfo> memGrpInfoDict;
-    static QDict<void>               expandAsDefinedDict;
-    static NamespaceDef             *globalScope;
+    static StringUnorderedSet        expandAsDefinedSet;
+    static NamespaceDefMutable      *globalScope;
     static QCString                  htmlFileExtension;
     static bool                      parseSourcesNeeded;
-    static QTime                     runningTime;
     static SearchIndexIntf          *searchIndex;
-    static QDict<DefinitionIntf>    *symbolMap;
+    static SymbolMap<Definition>     symbolMap;
     static QDict<Definition>        *clangUsrMap;
     static bool                      outputToWizard;
     static QDict<int>               *htmlDirMap;
-    static QCache<LookupInfo>       *lookupCache;
+    static Cache<std::string,LookupInfo> *lookupCache;
     static DirSDict                 *directories;
     static SDict<DirRelation>        dirRelations;
     static ParserManager            *parserManager;
     static bool                      suppressDocWarnings;
-    static Store                    *symbolStorage;
     static QCString                  objDBFileName;
     static QCString                  entryDBFileName;
     static QCString                  filterDBFileName;
@@ -138,16 +139,14 @@ class Doxygen
     static int                       subpageNestingLevel;
     static QCString                  spaces;
     static bool                      generatingXmlOutput;
-    static bool                      markdownSupport;
-    static GenericsSDict            *genericsDict;
-    static Preprocessor             *preprocessor;
+    static DefinesPerFileList        macroDefinitions;
+    static bool                      clangAssistedParsing;
 };
 
 void initDoxygen();
 void readConfiguration(int argc, char **argv);
 void checkConfiguration();
 void adjustConfiguration();
-void searchInputFiles(StringList &inputFiles);
 void parseInput();
 void generateOutput();
 void readAliases();
@@ -155,27 +154,16 @@ void readFormulaRepository(QCString dir, bool cmp = FALSE);
 void cleanUpDoxygen();
 int readFileOrDirectory(const char *s,
                         FileNameLinkedMap *fnDict,
-                        StringDict *exclDict,
-                        QStrList *patList,
-                        QStrList *exclPatList,
-                        StringList *resultList,
-                        StringDict *resultDict,
+                        StringUnorderedSet *exclSet,
+                        const StringVector *patList,
+                        const StringVector *exclPatList,
+                        StringVector *resultList,
+                        StringUnorderedSet *resultSet,
                         bool recursive,
                         bool errorIfNotExist=TRUE,
-                        QDict<void> *killDict = 0,
-                        QDict<void> *paths = 0
+                        StringUnorderedSet *killSet = 0,
+                        StringSet *paths = 0
                        );
-int readDir(QFileInfo *fi,
-            FileNameLinkedMap *fnDict,
-            StringDict  *exclDict,
-            QStrList *patList,
-            QStrList *exclPatList,
-            StringList *resultList,
-            StringDict *resultDict,
-            bool errorIfNotExist,
-            bool recursive,
-            QDict<void> *killDict
-           );
 void copyAndFilterFile(const char *fileName,BufStr &dest);
 
 #endif
