@@ -1,12 +1,12 @@
 /******************************************************************************
  *
- * 
+ *
  *
  * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -22,10 +22,13 @@
 
 #include "sortdict.h"
 #include "definition.h"
+#include "dirdef.h"
+#include "layout.h"
 
 class MemberList;
 class FileList;
-class ClassSDict;
+class ClassLinkedRefMap;
+class NamespaceLinkedRefMap;
 class FileDef;
 class ClassDef;
 class NamespaceDef;
@@ -33,18 +36,16 @@ class GroupList;
 class OutputList;
 class NamespaceSDict;
 class MemberGroupSDict;
-class MemberNameInfoSDict;
 class PageSDict;
 class PageDef;
 class DirDef;
-class DirList;
 class FTVHelp;
 class Entry;
 class MemberDef;
 class FTextStream;
 
 /** A model of a group of symbols. */
-class GroupDef : virtual public Definition
+class GroupDef : public DefinitionMutable, public Definition
 {
   public:
    ~GroupDef() {}
@@ -62,7 +63,7 @@ class GroupDef : virtual public Definition
     virtual void addGroup(const GroupDef *def) = 0;
     virtual void addPage(PageDef *def) = 0;
     virtual void addExample(const PageDef *def) = 0;
-    virtual void addDir(const DirDef *dd) = 0;
+    virtual void addDir(DirDef *dd) = 0;
     virtual bool insertMember(MemberDef *def,bool docOnly=FALSE) = 0;
     virtual void removeMember(MemberDef *md) = 0;
     virtual bool findGroup(const GroupDef *def) const = 0;
@@ -70,7 +71,7 @@ class GroupDef : virtual public Definition
     virtual void writeMemberPages(OutputList &ol) = 0;
     virtual void writeQuickMemberLinks(OutputList &ol,const MemberDef *currentMd) const = 0;
     virtual void writeTagFile(FTextStream &) = 0;
-    virtual int  numDocMembers() const = 0;
+    virtual size_t numDocMembers() const = 0;
     virtual bool isLinkableInProject() const = 0;
     virtual bool isLinkable() const = 0;
     virtual bool isASubGroup() const = 0;
@@ -95,11 +96,11 @@ class GroupDef : virtual public Definition
     virtual MemberGroupSDict *getMemberGroupSDict() const = 0;
 
     virtual FileList *      getFiles() const = 0;
-    virtual ClassSDict *    getClasses() const = 0;
-    virtual NamespaceSDict * getNamespaces() const = 0;
+    virtual ClassLinkedRefMap getClasses() const = 0;
+    virtual NamespaceLinkedRefMap getNamespaces() const = 0;
     virtual GroupList *     getSubGroups() const = 0;
     virtual PageSDict *     getPages() const = 0;
-    virtual DirList *       getDirs() const = 0;
+    virtual const DirList & getDirs() const = 0;
     virtual PageSDict *     getExamples() const = 0;
     virtual bool hasDetailedDescription() const = 0;
     virtual void sortSubGroups() = 0;
@@ -109,6 +110,14 @@ class GroupDef : virtual public Definition
 GroupDef *createGroupDef(const char *fileName,int line,const char *name,
                                 const char *title,const char *refFileName=0);
 
+// --- Cast functions
+
+GroupDef            *toGroupDef(Definition *d);
+const GroupDef      *toGroupDef(const Definition *d);
+
+// ------------------
+
+
 /** A sorted dictionary of GroupDef objects. */
 class GroupSDict : public SDict<GroupDef>
 {
