@@ -27,8 +27,8 @@
 #include "types.h"
 #include "definition.h"
 #include "arguments.h"
+#include "classdef.h"
 
-class ClassDef;
 class NamespaceDef;
 class GroupDef;
 class FileDef;
@@ -40,9 +40,10 @@ class GroupDef;
 class QTextStream;
 class QStrList;
 struct TagInfo;
+class MemberDefMutable;
 
 /** A model of a class/file/namespace member symbol. */
-class MemberDef : virtual public Definition
+class MemberDef : public Definition
 {
   public:
     virtual ~MemberDef() {}
@@ -54,6 +55,8 @@ class MemberDef : virtual public Definition
     virtual MemberDef *resolveAlias() = 0;
     virtual const MemberDef *resolveAlias() const = 0;
 
+    ClassDefMutable *getClassDefMutable() const;
+
     //-----------------------------------------------------------------------------------
     // ----  getters -----
     //-----------------------------------------------------------------------------------
@@ -73,29 +76,21 @@ class MemberDef : virtual public Definition
     virtual const QCString &initializer() const = 0;
     virtual int initializerLines() const = 0;
     virtual uint64 getMemberSpecifiers() const = 0;
-    virtual const MemberList *getSectionList() const = 0;
+    virtual const MemberList *getSectionList(const Definition *container) const = 0;
     virtual QCString    displayDefinition() const = 0;
 
     // scope query members
-    virtual const ClassDef *getClassDef() const = 0;
-    virtual ClassDef *getClassDef() = 0;
-
     virtual const FileDef *getFileDef() const = 0;
-    virtual FileDef *getFileDef() = 0;
-
+    virtual const ClassDef *getClassDef() const = 0;
     virtual const NamespaceDef* getNamespaceDef() const = 0;
-    virtual NamespaceDef* getNamespaceDef() = 0;
-
-    virtual const GroupDef *getGroupDef() const = 0;
-    virtual GroupDef *getGroupDef() = 0;
-
-    virtual ClassDef *accessorClass() const = 0;
+    virtual const ClassDef *accessorClass() const = 0;
 
     // grabbing the property read/write accessor names
     virtual const char *getReadAccessor() const = 0;
     virtual const char *getWriteAccessor() const = 0;
 
     // querying the grouping definition
+    virtual const GroupDef *getGroupDef() const = 0;
     virtual Grouping::GroupPri_t getGroupPri() const = 0;
     virtual const char *getGroupFileName() const = 0;
     virtual int getGroupStartLine() const = 0;
@@ -216,7 +211,6 @@ class MemberDef : virtual public Definition
     virtual bool isDocsForDefinition() const = 0;
     virtual const MemberDef *getEnumScope() const = 0;
     virtual const MemberList *enumFieldList() const = 0;
-    virtual void setEnumBaseType(const QCString &type) = 0;
     virtual QCString enumBaseType() const = 0;
 
     virtual bool hasExamples() const = 0;
@@ -225,7 +219,6 @@ class MemberDef : virtual public Definition
 
     // argument related members
     virtual const ArgumentList &argumentList() const = 0;
-    virtual ArgumentList &argumentList() = 0;
     virtual const ArgumentList &declArgumentList() const = 0;
     virtual const ArgumentList &templateArguments() const = 0;
     virtual const ArgumentLists &definitionTemplateParameterLists() const = 0;
@@ -280,15 +273,33 @@ class MemberDef : virtual public Definition
     virtual int getDeclLine() const = 0;
     virtual int getDeclColumn() const = 0;
 
+    virtual MemberDefMutable *createTemplateInstanceMember(const ArgumentList &formalArgs,
+               const std::unique_ptr<ArgumentList> &actualArgs) const = 0;
+    virtual void writeDeclaration(OutputList &ol,
+                 const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,
+                 bool inGroup, const ClassDef *inheritFrom=0,const char *inheritId=0) const = 0;
+    virtual void writeEnumDeclaration(OutputList &typeDecl, const ClassDef *cd,
+                const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd) const = 0;
+    virtual void detectUndocumentedParams(bool hasParamCommand,bool hasReturnCommand) const = 0;
+    virtual void warnIfUndocumented() const = 0;
+    virtual void warnIfUndocumentedParams() const = 0;
+
+    // TODO: this is not a getter, should be passed at construction
+    virtual void setMemberGroup(MemberGroup *grp) = 0;
+};
+
+class MemberDefMutable : public DefinitionMutable, public MemberDef
+{
+  public:
+
     //-----------------------------------------------------------------------------------
     // ----  setters -----
     //-----------------------------------------------------------------------------------
 
-
     // set functions
     virtual void setMemberType(MemberType t) = 0;
     virtual void setDefinition(const char *d) = 0;
-    virtual void setFileDef(FileDef *fd) = 0;
+    virtual void setFileDef(const FileDef *fd) = 0;
     virtual void setAnchor() = 0;
     virtual void setProtection(Protection p) = 0;
     virtual void setMemberSpecifiers(uint64 s) = 0;
@@ -296,9 +307,9 @@ class MemberDef : virtual public Definition
     virtual void setInitializer(const char *i) = 0;
     virtual void setBitfields(const char *s) = 0;
     virtual void setMaxInitLines(int lines) = 0;
-    virtual void setMemberClass(ClassDef *cd) = 0;
-    virtual void setSectionList(MemberList *sl) = 0;
-    virtual void setGroupDef(GroupDef *gd,Grouping::GroupPri_t pri,
+    virtual void setMemberClass(const ClassDef *cd) = 0;
+    virtual void setSectionList(const Definition *container,MemberList *sl) = 0;
+    virtual void setGroupDef(const GroupDef *gd,Grouping::GroupPri_t pri,
                      const QCString &fileName,int startLine,bool hasDocs,
                      MemberDef *member=0) = 0;
     virtual void setReadAccessor(const char *r) = 0;
@@ -311,6 +322,7 @@ class MemberDef : virtual public Definition
     virtual void setTagInfo(const TagInfo *i) = 0;
     virtual void setArgsString(const char *as) = 0;
     virtual void incrementFlowKeyWordCount() = 0;
+    virtual void setEnumBaseType(const QCString &type) = 0;
 
     // relation to other members
     virtual void setReimplements(MemberDef *md) = 0;
@@ -336,16 +348,16 @@ class MemberDef : virtual public Definition
     // argument related members
     virtual void moveArgumentList(std::unique_ptr<ArgumentList> al) = 0;
     virtual void moveDeclArgumentList(std::unique_ptr<ArgumentList> al) = 0;
+    virtual void resolveUnnamedParameters(const MemberDef *md) = 0;
     virtual void setDefinitionTemplateParameterLists(const ArgumentLists &lists) = 0;
     virtual void setTypeConstraints(const ArgumentList &al) = 0;
     virtual void setType(const char *t) = 0;
     virtual void setAccessorType(ClassDef *cd,const char *t) = 0;
 
     // namespace related members
-    virtual void setNamespace(NamespaceDef *nd) = 0;
+    virtual void setNamespace(const NamespaceDef *nd) = 0;
 
     // member group related members
-    virtual void setMemberGroup(MemberGroup *grp) = 0;
     virtual void setMemberGroupId(int id) = 0;
     virtual void makeImplementationDetail() = 0;
 
@@ -387,8 +399,6 @@ class MemberDef : virtual public Definition
     // --- actions ----
     //-----------------------------------------------------------------------------------
 
-    virtual MemberDef *createTemplateInstanceMember(const ArgumentList &formalArgs,
-               const std::unique_ptr<ArgumentList> &actualArgs) const = 0;
     virtual void findSectionsInDocumentation() = 0;
     virtual void addToSearchIndex() const = 0;
 
@@ -396,32 +406,40 @@ class MemberDef : virtual public Definition
     // --- write output ----
     //-----------------------------------------------------------------------------------
 
-    virtual void writeDeclaration(OutputList &ol,
-                 const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,
-                 bool inGroup, const ClassDef *inheritFrom=0,const char *inheritId=0) const = 0;
     virtual void writeDocumentation(const MemberList *ml,int memCount,int memTotal,OutputList &ol,
                  const char *scopeName,const Definition *container,
                  bool inGroup,bool showEnumValues=FALSE,bool
                  showInline=FALSE) const = 0;
     virtual void writeMemberDocSimple(OutputList &ol,const Definition *container) const = 0;
-    virtual void writeEnumDeclaration(OutputList &typeDecl, const ClassDef *cd,
-                const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd) const = 0;
     virtual void writeTagFile(FTextStream &) const = 0;
     virtual void writeLink(OutputList &ol,
                  const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,
                  bool onlyText=FALSE) const = 0;
 
     // write helpers
-    virtual void warnIfUndocumented() const = 0;
-    virtual void warnIfUndocumentedParams() const = 0;
-    virtual void detectUndocumentedParams(bool hasParamCommand,bool hasReturnCommand) const = 0;
     virtual void setAnonymousUsed() const = 0;
     virtual void setFromAnonymousScope(bool b) const = 0;
 
 };
 
+inline ClassDefMutable *MemberDef::getClassDefMutable() const
+{
+  return toClassDefMutable(getClassDef());
+}
+
+// --- Cast functions
+
+MemberDef            *toMemberDef(Definition *d);
+MemberDef            *toMemberDef(DefinitionMutable *d);
+const MemberDef      *toMemberDef(const Definition *d);
+MemberDefMutable     *toMemberDefMutable(Definition *d);
+MemberDefMutable     *toMemberDefMutable(const Definition *d);
+
+//------------------------------------------------------------------------
+
+
 /** Factory method to create a new instance of a MemberDef */
-MemberDef *createMemberDef(const char *defFileName,int defLine,int defColumn,
+MemberDefMutable *createMemberDef(const char *defFileName,int defLine,int defColumn,
               const char *type,const char *name,const char *args,
               const char *excp,Protection prot,Specifier virt,bool stat,
               Relationship related,MemberType t,const ArgumentList &tal,
@@ -429,6 +447,7 @@ MemberDef *createMemberDef(const char *defFileName,int defLine,int defColumn,
 
 MemberDef *createMemberDefAlias(const Definition *newScope,const MemberDef *aliasMd);
 
-void combineDeclarationAndDefinition(MemberDef *mdec,MemberDef *mdef);
+void combineDeclarationAndDefinition(MemberDefMutable *mdec,MemberDefMutable *mdef);
+void addDocCrossReference(MemberDefMutable *src,MemberDefMutable *dst);
 
 #endif
