@@ -37,6 +37,8 @@
 #include "filename.h"
 #include "membername.h"
 #include "resourcemgr.h"
+#include "namespacedef.h"
+#include "classdef.h"
 
 // file format: (all multi-byte values are stored in big endian format)
 //   4 byte header
@@ -49,49 +51,41 @@
 //                 (4 bytes index to url string + 4 bytes frequency counter)
 //   for each url: a \0 terminated string
 
-const int numIndexEntries = 256*256;
+const size_t numIndexEntries = 256*256;
 
 //--------------------------------------------------------------------
 
-IndexWord::IndexWord(const char *word) : m_word(word), m_urls(17)
+IndexWord::IndexWord(QCString word) : m_word(word)
 {
-  m_urls.setAutoDelete(TRUE);
   //printf("IndexWord::IndexWord(%s)\n",word);
 }
 
 void IndexWord::addUrlIndex(int idx,bool hiPriority)
 {
   //printf("IndexWord::addUrlIndex(%d,%d)\n",idx,hiPriority);
-  URLInfo *ui = m_urls.find(idx);
-  if (ui==0)
+  auto it = m_urls.find(idx);
+  if (it==m_urls.end())
   {
     //printf("URLInfo::URLInfo(%d)\n",idx);
-    ui=new URLInfo(idx,0);
-    m_urls.insert(idx,ui);
+    it = m_urls.insert(std::make_pair(idx,URLInfo(idx,0))).first;
   }
-  ui->freq+=2;
-  if (hiPriority) ui->freq|=1; // mark as high priority document
+  it->second.freq+=2;
+  if (hiPriority) it->second.freq|=1; // mark as high priority document
 }
 
 //--------------------------------------------------------------------
 
-SearchIndex::SearchIndex() : SearchIndexIntf(Internal),
-      m_words(328829), m_index(numIndexEntries), m_url2IdMap(10007), m_urls(10007), m_urlIndex(-1)
+SearchIndex::SearchIndex() : SearchIndexIntf(Internal)
 {
-  int i;
-  m_words.setAutoDelete(TRUE);
-  m_url2IdMap.setAutoDelete(TRUE);
-  m_urls.setAutoDelete(TRUE);
-  m_index.setAutoDelete(TRUE);
-  for (i=0;i<numIndexEntries;i++) m_index.insert(i,new QList<IndexWord>);
+  m_index.resize(numIndexEntries);
 }
 
-void SearchIndex::setCurrentDoc(Definition *ctx,const char *anchor,bool isSourceFile)
+void SearchIndex::setCurrentDoc(const Definition *ctx,const char *anchor,bool isSourceFile)
 {
   if (ctx==0) return;
   assert(!isSourceFile || ctx->definitionType()==Definition::TypeFile);
   //printf("SearchIndex::setCurrentDoc(%s,%s,%s)\n",name,baseName,anchor);
-  QCString url=isSourceFile ? ((FileDef*)ctx)->getSourceFileBase() : ctx->getOutputFileBase();
+  QCString url=isSourceFile ? (toFileDef(ctx))->getSourceFileBase() : ctx->getOutputFileBase();
   url+=Config_getString(HTML_FILE_EXTENSION);
   QCString baseUrl = url;
   if (anchor) url+=QCString("#")+anchor;
@@ -99,7 +93,7 @@ void SearchIndex::setCurrentDoc(Definition *ctx,const char *anchor,bool isSource
   QCString name=ctx->qualifiedName();
   if (ctx->definitionType()==Definition::TypeMember)
   {
-    MemberDef *md = (MemberDef *)ctx;
+    const MemberDef *md = toMemberDef(ctx);
     name.prepend((md->getLanguage()==SrcLangExt_Fortran  ?
                  theTranslator->trSubprogram(TRUE,TRUE) :
                  theTranslator->trMember(TRUE,TRUE))+" ");
@@ -116,8 +110,8 @@ void SearchIndex::setCurrentDoc(Definition *ctx,const char *anchor,bool isSource
     {
       case Definition::TypePage:
         {
-          PageDef *pd = (PageDef *)ctx;
-          if (!pd->title().isEmpty())
+          const PageDef *pd = toPageDef(ctx);
+          if (pd->hasTitle())
           {
             name = theTranslator->trPage(TRUE,TRUE)+" "+pd->title();
           }
@@ -129,7 +123,7 @@ void SearchIndex::setCurrentDoc(Definition *ctx,const char *anchor,bool isSource
         break;
       case Definition::TypeClass:
         {
-          ClassDef *cd = (ClassDef *)ctx;
+          const ClassDef *cd = toClassDef(ctx);
           name.prepend(cd->compoundTypeString()+" ");
         }
         break;
@@ -151,7 +145,7 @@ void SearchIndex::setCurrentDoc(Definition *ctx,const char *anchor,bool isSource
         break;
       case Definition::TypeGroup:
         {
-          GroupDef *gd = (GroupDef *)ctx;
+          const GroupDef *gd = toGroupDef(ctx);
           if (gd->groupTitle())
           {
             name = theTranslator->trGroup(TRUE,TRUE)+" "+gd->groupTitle();
@@ -167,16 +161,16 @@ void SearchIndex::setCurrentDoc(Definition *ctx,const char *anchor,bool isSource
     }
   }
 
-  int *pIndex = m_url2IdMap.find(baseUrl);
-  if (pIndex==0)
+  auto it = m_url2IdMap.find(baseUrl.str());
+  if (it == m_url2IdMap.end())
   {
     ++m_urlIndex;
-    m_url2IdMap.insert(baseUrl,new int(m_urlIndex));
-    m_urls.insert(m_urlIndex,new URL(name,url));
+    m_url2IdMap.insert(std::make_pair(baseUrl.str(),m_urlIndex));
+    m_urls.insert(std::make_pair(m_urlIndex,URL(name,url)));
   }
   else
   {
-    m_urls.insert(*pIndex,new URL(name,url));
+    m_urls.insert(std::make_pair(it->second,URL(name,url)));
   }
 }
 
@@ -208,17 +202,16 @@ void SearchIndex::addWord(const char *word,bool hiPriority,bool recurse)
   if (word==0 || word[0]=='\0') return;
   QCString wStr = QCString(word).lower();
   //printf("SearchIndex::addWord(%s,%d) wStr=%s\n",word,hiPriority,wStr.data());
-  IndexWord *w = m_words[wStr];
-  if (w==0)
+  int idx=charsToIndex(wStr);
+  if (idx<0 || idx>=static_cast<int>(m_index.size())) return;
+  auto it = m_words.find(wStr.str());
+  if (it==m_words.end())
   {
-    int idx=charsToIndex(wStr);
     //fprintf(stderr,"addWord(%s) at index %d\n",word,idx);
-    if (idx<0) return;
-    w = new IndexWord(wStr);
-    m_index[idx]->append(w);
-    m_words.insert(wStr,w);
+    m_index[idx].push_back(IndexWord(wStr));
+    it = m_words.insert({ wStr.str(), static_cast<int>(m_index[idx].size())-1 }).first;
   }
-  w->addUrlIndex(m_urlIndex,hiPriority);
+  m_index[idx][it->second].addUrlIndex(m_urlIndex,hiPriority);
   int i;
   bool found=FALSE;
   if (!recurse) // the first time we check if we can strip the prefix
@@ -244,12 +237,12 @@ void SearchIndex::addWord(const char *word,bool hiPriority)
   addWord(word,hiPriority,FALSE);
 }
 
-static void writeInt(QFile &f,int index)
+static void writeInt(QFile &f,size_t index)
 {
-  f.putch(((uint)index)>>24);
-  f.putch((((uint)index)>>16)&0xff);
-  f.putch((((uint)index)>>8)&0xff);
-  f.putch(((uint)index)&0xff);
+  f.putch(static_cast<int>(index>>24));
+  f.putch(static_cast<int>((index>>16)&0xff));
+  f.putch(static_cast<int>((index>>8)&0xff));
+  f.putch(static_cast<int>(index&0xff));
 }
 
 static void writeString(QFile &f,const char *s)
@@ -261,21 +254,19 @@ static void writeString(QFile &f,const char *s)
 
 void SearchIndex::write(const char *fileName)
 {
-  int i;
-  int size=4; // for the header
+  size_t i;
+  size_t size=4; // for the header
   size+=4*numIndexEntries; // for the index
-  int wordsOffset = size;
+  size_t wordsOffset = size;
   // first pass: compute the size of the wordlist
   for (i=0;i<numIndexEntries;i++)
   {
-    QList<IndexWord> *wlist = m_index[i];
-    if (!wlist->isEmpty())
+    const auto &wlist = m_index[i];
+    if (!wlist.empty())
     {
-      QListIterator<IndexWord> iwi(*wlist);
-      IndexWord *iw;
-      for (iwi.toFirst();(iw=iwi.current());++iwi)
+      for (const auto &iw : wlist)
       {
-        int ws = iw->word().length()+1;
+        int ws = iw.word().length()+1;
         size+=ws+4; // word + url info list offset
       }
       size+=1; // zero list terminator
@@ -283,19 +274,17 @@ void SearchIndex::write(const char *fileName)
   }
 
   // second pass: compute the offsets in the index
-  int indexOffsets[numIndexEntries];
-  int offset=wordsOffset;
+  size_t indexOffsets[numIndexEntries];
+  size_t offset=wordsOffset;
   for (i=0;i<numIndexEntries;i++)
   {
-    QList<IndexWord> *wlist = m_index[i];
-    if (!wlist->isEmpty())
+    const auto &wlist = m_index[i];
+    if (!wlist.empty())
     {
       indexOffsets[i]=offset;
-      QListIterator<IndexWord> iwi(*wlist);
-      IndexWord *iw;
-      for (iwi.toFirst();(iw=iwi.current());++iwi)
+      for (const auto &iw : wlist)
       {
-        offset+= iw->word().length()+1;
+        offset+= iw.word().length()+1;
         offset+=4; // word + offset to url info array
       }
       offset+=1; // zero list terminator
@@ -305,42 +294,36 @@ void SearchIndex::write(const char *fileName)
       indexOffsets[i]=0;
     }
   }
-  int padding = size;
+  size_t padding = size;
   size = (size+3)&~3; // round up to 4 byte boundary
   padding = size - padding;
 
-  //int statsOffset = size;
-  //IndexWord *iw;
-  int *wordStatOffsets = new int[m_words.count()];
+  std::vector<size_t> wordStatOffsets(m_words.size());
 
   int count=0;
 
   // third pass: compute offset to stats info for each word
   for (i=0;i<numIndexEntries;i++)
   {
-    QList<IndexWord> *wlist = m_index[i];
-    if (!wlist->isEmpty())
+    const auto &wlist = m_index[i];
+    if (!wlist.empty())
     {
-      QListIterator<IndexWord> iwi(*wlist);
-      IndexWord *iw;
-      for (iwi.toFirst();(iw=iwi.current());++iwi)
+      for (const auto &iw : wlist)
       {
         //printf("wordStatOffsets[%d]=%d\n",count,size);
         wordStatOffsets[count++] = size;
-        size+=4+iw->urls().count()*8; // count + (url_index,freq) per url
+        size+=4 + iw.urls().size() * 8; // count + (url_index,freq) per url
       }
     }
   }
-  int *urlOffsets = new int[m_urls.count()];
-  //int urlsOffset = size;
-  QIntDictIterator<URL> udi(m_urls);
-  URL *url;
-  for (udi.toFirst();(url=udi.current());++udi)
+  std::vector<size_t> urlOffsets(m_urls.size());
+  for (const auto &udi : m_urls)
   {
-    urlOffsets[udi.currentKey()]=size;
-    size+=url->name.length()+1+
-          url->url.length()+1;
+    urlOffsets[udi.first]=size;
+    size+=udi.second.name.length()+1+
+          udi.second.url.length()+1;
   }
+
   //printf("Total size %x bytes (word=%x stats=%x urls=%x)\n",size,wordsOffset,statsOffset,urlsOffset);
   QFile f(fileName);
   if (f.open(IO_WriteOnly))
@@ -356,14 +339,12 @@ void SearchIndex::write(const char *fileName)
     count=0;
     for (i=0;i<numIndexEntries;i++)
     {
-      QList<IndexWord> *wlist = m_index[i];
-      if (!wlist->isEmpty())
+      const auto &wlist = m_index[i];
+      if (!wlist.empty())
       {
-        QListIterator<IndexWord> iwi(*wlist);
-        IndexWord *iw;
-        for (iwi.toFirst();(iw=iwi.current());++iwi)
+        for (const auto &iw : wlist)
         {
-          writeString(f,iw->word());
+          writeString(f,iw.word());
           writeInt(f,wordStatOffsets[count++]);
         }
         f.putch(0);
@@ -374,37 +355,29 @@ void SearchIndex::write(const char *fileName)
     // write word statistics
     for (i=0;i<numIndexEntries;i++)
     {
-      QList<IndexWord> *wlist = m_index[i];
-      if (!wlist->isEmpty())
+      const auto &wlist = m_index[i];
+      if (!wlist.empty())
       {
-        QListIterator<IndexWord> iwi(*wlist);
-        IndexWord *iw;
-        for (iwi.toFirst();(iw=iwi.current());++iwi)
+        for (const auto &iw : wlist)
         {
-          int numUrls = iw->urls().count();
+          size_t numUrls = iw.urls().size();
           writeInt(f,numUrls);
-          QIntDictIterator<URLInfo> uli(iw->urls());
-          URLInfo *ui;
-          for (uli.toFirst();(ui=uli.current());++uli)
+          for (const auto &ui : iw.urls())
           {
-            writeInt(f,urlOffsets[ui->urlIdx]);
-            writeInt(f,ui->freq);
+            writeInt(f,urlOffsets[ui.second.urlIdx]);
+            writeInt(f,ui.second.freq);
           }
         }
       }
     }
     // write urls
-    QIntDictIterator<URL> udi(m_urls);
-    URL *url;
-    for (udi.toFirst();(url=udi.current());++udi)
+    for (const auto &udi : m_urls)
     {
-      writeString(f,url->name);
-      writeString(f,url->url);
+      writeString(f,udi.second.name);
+      writeString(f,udi.second.url);
     }
   }
 
-  delete[] urlOffsets;
-  delete[] wordStatOffsets;
 }
 
 
@@ -424,29 +397,19 @@ struct SearchDocEntry
 
 struct SearchIndexExternal::Private
 {
-  Private() : docEntries(12251) {}
-  SDict<SearchDocEntry> docEntries;
-  SearchDocEntry *current;
+  std::map<std::string,SearchDocEntry> docEntries;
+  SearchDocEntry *current = 0;
 };
 
-SearchIndexExternal::SearchIndexExternal() : SearchIndexIntf(External)
+SearchIndexExternal::SearchIndexExternal() : SearchIndexIntf(External), p(std::make_unique<Private>())
 {
-  p = new SearchIndexExternal::Private;
-  p->docEntries.setAutoDelete(TRUE);
-  p->current=0;
 }
 
-SearchIndexExternal::~SearchIndexExternal()
-{
-  //printf("p->docEntries.count()=%d\n",p->docEntries.count());
-  delete p;
-}
-
-static QCString definitionToName(Definition *ctx)
+static QCString definitionToName(const Definition *ctx)
 {
   if (ctx && ctx->definitionType()==Definition::TypeMember)
   {
-    MemberDef *md = (MemberDef*)ctx;
+    const MemberDef *md = toMemberDef(ctx);
     if (md->isFunction())
       return "function";
     else if (md->isSlot())
@@ -477,7 +440,7 @@ static QCString definitionToName(Definition *ctx)
     switch(ctx->definitionType())
     {
       case Definition::TypeClass:
-        return ((ClassDef*)ctx)->compoundTypeString();
+        return (toClassDef(ctx))->compoundTypeString();
       case Definition::TypeFile:
         return "file";
       case Definition::TypeNamespace:
@@ -497,31 +460,30 @@ static QCString definitionToName(Definition *ctx)
   return "unknown";
 }
 
-void SearchIndexExternal::setCurrentDoc(Definition *ctx,const char *anchor,bool isSourceFile)
+void SearchIndexExternal::setCurrentDoc(const Definition *ctx,const char *anchor,bool isSourceFile)
 {
-  QCString extId = stripPath(Config_getString(EXTERNAL_SEARCH_ID));
-  QCString baseName = isSourceFile ? ((FileDef*)ctx)->getSourceFileBase() : ctx->getOutputFileBase();
+  static QCString extId = stripPath(Config_getString(EXTERNAL_SEARCH_ID));
+  QCString baseName = isSourceFile ? (toFileDef(ctx))->getSourceFileBase() : ctx->getOutputFileBase();
   QCString url = baseName + Doxygen::htmlFileExtension;
   if (anchor) url+=QCString("#")+anchor;
   QCString key = extId+";"+url;
 
-  p->current = p->docEntries.find(key);
-  //printf("setCurrentDoc(url=%s,isSourceFile=%d) current=%p\n",url.data(),isSourceFile,p->current);
-  if (!p->current)
+  auto it = p->docEntries.find(key.str());
+  if (it == p->docEntries.end())
   {
-    SearchDocEntry *e = new SearchDocEntry;
-    e->type = isSourceFile ? QCString("source") : definitionToName(ctx);
-    e->name = ctx->qualifiedName();
+    SearchDocEntry e;
+    e.type = isSourceFile ? QCString("source") : definitionToName(ctx);
+    e.name = ctx->qualifiedName();
     if (ctx->definitionType()==Definition::TypeMember)
     {
-      e->args = ((MemberDef*)ctx)->argsString();
+      e.args = (toMemberDef(ctx))->argsString();
     }
-    e->extId = extId;
-    e->url  = url;
-    p->current = e;
-    p->docEntries.append(key,e);
+    e.extId = extId;
+    e.url  = url;
+    it = p->docEntries.insert({key.str(),e}).first;
     //printf("searchIndexExt %s : %s\n",e->name.data(),e->url.data());
   }
+  p->current = &it->second;
 }
 
 void SearchIndexExternal::addWord(const char *word,bool hiPriority)
@@ -541,26 +503,25 @@ void SearchIndexExternal::write(const char *fileName)
     FTextStream t(&f);
     t << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
     t << "<add>" << endl;
-    SDict<SearchDocEntry>::Iterator it(p->docEntries);
-    SearchDocEntry *doc;
-    for (it.toFirst();(doc=it.current());++it)
+    for (auto &kv : p->docEntries)
     {
-      doc->normalText.addChar(0);    // make sure buffer ends with a 0 terminator
-      doc->importantText.addChar(0); // make sure buffer ends with a 0 terminator
+      SearchDocEntry &doc = kv.second;
+      doc.normalText.addChar(0);    // make sure buffer ends with a 0 terminator
+      doc.importantText.addChar(0); // make sure buffer ends with a 0 terminator
       t << "  <doc>" << endl;
-      t << "    <field name=\"type\">"     << doc->type << "</field>" << endl;
-      t << "    <field name=\"name\">"     << convertToXML(doc->name) << "</field>" << endl;
-      if (!doc->args.isEmpty())
+      t << "    <field name=\"type\">"     << doc.type << "</field>" << endl;
+      t << "    <field name=\"name\">"     << convertToXML(doc.name) << "</field>" << endl;
+      if (!doc.args.isEmpty())
       {
-        t << "    <field name=\"args\">"     << convertToXML(doc->args) << "</field>" << endl;
+        t << "    <field name=\"args\">"     << convertToXML(doc.args) << "</field>" << endl;
       }
-      if (!doc->extId.isEmpty())
+      if (!doc.extId.isEmpty())
       {
-        t << "    <field name=\"tag\">"      << convertToXML(doc->extId)  << "</field>" << endl;
+        t << "    <field name=\"tag\">"      << convertToXML(doc.extId)  << "</field>" << endl;
       }
-      t << "    <field name=\"url\">"      << convertToXML(doc->url)  << "</field>" << endl;
-      t << "    <field name=\"keywords\">" << convertToXML(doc->importantText.get())  << "</field>" << endl;
-      t << "    <field name=\"text\">"     << convertToXML(doc->normalText.get())     << "</field>" << endl;
+      t << "    <field name=\"url\">"      << convertToXML(doc.url)  << "</field>" << endl;
+      t << "    <field name=\"keywords\">" << convertToXML(doc.importantText.get())  << "</field>" << endl;
+      t << "    <field name=\"text\">"     << convertToXML(doc.normalText.get())     << "</field>" << endl;
       t << "  </doc>" << endl;
     }
     t << "</add>" << endl;
@@ -574,25 +535,17 @@ void SearchIndexExternal::write(const char *fileName)
 //---------------------------------------------------------------------------
 // the following part is for the javascript based search engine
 
-#include "memberdef.h"
-#include "namespacedef.h"
-#include "pagedef.h"
-#include "classdef.h"
-#include "filedef.h"
-#include "language.h"
-#include "doxygen.h"
-#include "message.h"
 
 static SearchIndexInfo g_searchIndexInfo[NUM_SEARCH_INDICES];
 
-static void addMemberToSearchIndex(MemberDef *md)
+static void addMemberToSearchIndex(const MemberDef *md)
 {
   static bool hideFriendCompounds = Config_getBool(HIDE_FRIEND_COMPOUNDS);
   bool isLinkable = md->isLinkable();
-  ClassDef *cd=0;
-  NamespaceDef *nd=0;
-  FileDef *fd=0;
-  GroupDef *gd=0;
+  const ClassDef *cd=0;
+  const NamespaceDef *nd=0;
+  const FileDef *fd=0;
+  const GroupDef *gd=0;
   if (isLinkable &&
       (
        ((cd=md->getClassDef()) && cd->isLinkable() && cd->templateMaster()==0) ||
@@ -731,7 +684,7 @@ static QCString searchId(const QCString &s)
   return result;
 }
 
-void createJavascriptSearchIndex()
+void createJavaScriptSearchIndex()
 {
   bool sliceOpt = Config_getBool(OPTIMIZE_OUTPUT_SLICE);
 
@@ -785,101 +738,85 @@ void createJavascriptSearchIndex()
   // add symbols to letter -> symbol list map
 
   // index classes
-  ClassSDict::Iterator cli(*Doxygen::classSDict);
-  ClassDef *cd;
-  for (;(cd=cli.current());++cli)
+  for (const auto &cd : *Doxygen::classLinkedMap)
   {
     uint letter = getUtf8CodeToLower(cd->localName(),0);
     if (cd->isLinkable() && isId(letter))
     {
-      g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,cd);
+      g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,cd.get());
       if (sliceOpt)
       {
         if (cd->compoundType()==ClassDef::Interface)
         {
-          g_searchIndexInfo[SEARCH_INDEX_INTERFACES].symbolList.append(letter,cd);
+          g_searchIndexInfo[SEARCH_INDEX_INTERFACES].symbolList.append(letter,cd.get());
         }
         else if (cd->compoundType()==ClassDef::Struct)
         {
-          g_searchIndexInfo[SEARCH_INDEX_STRUCTS].symbolList.append(letter,cd);
+          g_searchIndexInfo[SEARCH_INDEX_STRUCTS].symbolList.append(letter,cd.get());
         }
         else if (cd->compoundType()==ClassDef::Exception)
         {
-          g_searchIndexInfo[SEARCH_INDEX_EXCEPTIONS].symbolList.append(letter,cd);
+          g_searchIndexInfo[SEARCH_INDEX_EXCEPTIONS].symbolList.append(letter,cd.get());
         }
         else // cd->compoundType()==ClassDef::Class
         {
-          g_searchIndexInfo[SEARCH_INDEX_CLASSES].symbolList.append(letter,cd);
+          g_searchIndexInfo[SEARCH_INDEX_CLASSES].symbolList.append(letter,cd.get());
         }
       }
       else // non slice optimisation: group all types under classes
       {
-        g_searchIndexInfo[SEARCH_INDEX_CLASSES].symbolList.append(letter,cd);
+        g_searchIndexInfo[SEARCH_INDEX_CLASSES].symbolList.append(letter,cd.get());
       }
     }
   }
 
   // index namespaces
-  NamespaceSDict::Iterator nli(*Doxygen::namespaceSDict);
-  NamespaceDef *nd;
-  for (;(nd=nli.current());++nli)
+  for (const auto &nd : *Doxygen::namespaceLinkedMap)
   {
     uint letter = getUtf8CodeToLower(nd->name(),0);
     if (nd->isLinkable() && isId(letter))
     {
-      g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,nd);
-      g_searchIndexInfo[SEARCH_INDEX_NAMESPACES].symbolList.append(letter,nd);
+      g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,nd.get());
+      g_searchIndexInfo[SEARCH_INDEX_NAMESPACES].symbolList.append(letter,nd.get());
     }
   }
 
   // index files
-  FileNameListIterator fnli(*Doxygen::inputNameList);
-  FileName *fn;
-  for (;(fn=fnli.current());++fnli)
+  for (const auto &fn : *Doxygen::inputNameLinkedMap)
   {
-    FileNameIterator fni(*fn);
-    FileDef *fd;
-    for (;(fd=fni.current());++fni)
+    for (const auto &fd : *fn)
     {
       uint letter = getUtf8CodeToLower(fd->name(),0);
       if (fd->isLinkable() && isId(letter))
       {
-        g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,fd);
-        g_searchIndexInfo[SEARCH_INDEX_FILES].symbolList.append(letter,fd);
+        g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,fd.get());
+        g_searchIndexInfo[SEARCH_INDEX_FILES].symbolList.append(letter,fd.get());
       }
     }
   }
 
   // index class members
   {
-    MemberNameSDict::Iterator mnli(*Doxygen::memberNameSDict);
-    MemberName *mn;
     // for each member name
-    for (mnli.toFirst();(mn=mnli.current());++mnli)
+    for (const auto &mn : *Doxygen::memberNameLinkedMap)
     {
-      MemberDef *md;
-      MemberNameIterator mni(*mn);
       // for each member definition
-      for (mni.toFirst();(md=mni.current());++mni)
+      for (const auto &md : *mn)
       {
-        addMemberToSearchIndex(md);
+        addMemberToSearchIndex(md.get());
       }
     }
   }
 
   // index file/namespace members
   {
-    MemberNameSDict::Iterator fnli(*Doxygen::functionNameSDict);
-    MemberName *mn;
     // for each member name
-    for (fnli.toFirst();(mn=fnli.current());++fnli)
+    for (const auto &mn : *Doxygen::functionNameLinkedMap)
     {
-      MemberDef *md;
-      MemberNameIterator mni(*mn);
       // for each member definition
-      for (mni.toFirst();(md=mni.current());++mni)
+      for (const auto &md : *mn)
       {
-        addMemberToSearchIndex(md);
+        addMemberToSearchIndex(md.get());
       }
     }
   }
@@ -953,9 +890,10 @@ void createJavascriptSearchIndex()
   }
 }
 
-void writeJavascriptSearchIndex()
+void writeJavaScriptSearchIndex()
 {
   int i;
+  int cnt = 0;
   // write index files
   QCString searchDirName = Config_getString(HTML_OUTPUT)+"/search";
 
@@ -969,7 +907,7 @@ void writeJavascriptSearchIndex()
       QCString baseName;
       baseName.sprintf("%s_%x",g_searchIndexInfo[i].name.data(),p);
 
-      QCString fileName = searchDirName + "/"+baseName+".html";
+      QCString fileName = searchDirName + "/"+baseName+Doxygen::htmlFileExtension;
       QCString dataFileName = searchDirName + "/"+baseName+".js";
 
       QFile outFile(fileName);
@@ -981,9 +919,10 @@ void writeJavascriptSearchIndex()
 
           t << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\""
             " \"https://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">" << endl;
-          t << "<html><head><title></title>" << endl;
+          t << "<html xmlns=\"http://www.w3.org/1999/xhtml\">" << endl;
+          t << "<head><title></title>" << endl;
           t << "<meta http-equiv=\"Content-Type\" content=\"text/xhtml;charset=UTF-8\"/>" << endl;
-          t << "<meta name=\"generator\" content=\"Doxygen " << versionString << "\"/>" << endl;
+          t << "<meta name=\"generator\" content=\"Doxygen " << getDoxygenVersion() << "\"/>" << endl;
           t << "<link rel=\"stylesheet\" type=\"text/css\" href=\"search.css\"/>" << endl;
           t << "<script type=\"text/javascript\" src=\"" << baseName << ".js\"></script>" << endl;
           t << "<script type=\"text/javascript\" src=\"search.js\"></script>" << endl;
@@ -992,24 +931,30 @@ void writeJavascriptSearchIndex()
           t << "<div id=\"SRIndex\">" << endl;
           t << "<div class=\"SRStatus\" id=\"Loading\">" << theTranslator->trLoading() << "</div>" << endl;
           t << "<div id=\"SRResults\"></div>" << endl; // here the results will be inserted
-          t << "<script type=\"text/javascript\"><!--" << endl;
-					t << "/* @license magnet:?xt=urn:btih:cf05388f2679ee054f2beb29a391d25f4e673ac3&amp;dn=gpl-2.0.txt GPL-v2 */\n";
+          t << "<script type=\"text/javascript\">" << endl;
+          t << "/* @license magnet:?xt=urn:btih:cf05388f2679ee054f2beb29a391d25f4e673ac3&amp;dn=gpl-2.0.txt GPL-v2 */\n";
           t << "createResults();" << endl; // this function will insert the results
-					t << "/* @license-end */\n";
-          t << "--></script>" << endl;
+          t << "/* @license-end */\n";
+          t << "</script>" << endl;
           t << "<div class=\"SRStatus\" id=\"Searching\">"
             << theTranslator->trSearching() << "</div>" << endl;
           t << "<div class=\"SRStatus\" id=\"NoMatches\">"
             << theTranslator->trNoMatches() << "</div>" << endl;
 
-          t << "<script type=\"text/javascript\"><!--" << endl;
-					t << "/* @license magnet:?xt=urn:btih:cf05388f2679ee054f2beb29a391d25f4e673ac3&amp;dn=gpl-2.0.txt GPL-v2 */\n";
+          t << "<script type=\"text/javascript\">" << endl;
+          t << "/* @license magnet:?xt=urn:btih:cf05388f2679ee054f2beb29a391d25f4e673ac3&amp;dn=gpl-2.0.txt GPL-v2 */\n";
           t << "document.getElementById(\"Loading\").style.display=\"none\";" << endl;
           t << "document.getElementById(\"NoMatches\").style.display=\"none\";" << endl;
           t << "var searchResults = new SearchResults(\"searchResults\");" << endl;
           t << "searchResults.Search();" << endl;
-					t << "/* @license-end */\n";
-          t << "--></script>" << endl;
+          t << "window.addEventListener(\"message\", function(event) {" << endl;
+          t << "  if (event.data == \"take_focus\") {" << endl;
+          t << "    var elem = searchResults.NavNext(0);" << endl;
+          t << "    if (elem) elem.focus();" << endl;
+          t << "  }" << endl;
+          t << "});" << endl;
+          t << "/* @license-end */\n";
+          t << "</script>" << endl;
           t << "</div>" << endl; // SRIndex
           t << "</body>" << endl;
           t << "</html>" << endl;
@@ -1035,7 +980,7 @@ void writeJavascriptSearchIndex()
         int itemCount=0;
         for (li.toFirst();(dl=li.current());++li)
         {
-          Definition *d = dl->getFirst();
+          const Definition *d = dl->getFirst();
 
           if (!firstEntry)
           {
@@ -1043,17 +988,15 @@ void writeJavascriptSearchIndex()
           }
           firstEntry=FALSE;
 
-          ti << "  ['" << dl->id() << "',['" << convertToXML(dl->name()) << "',[";
+          ti << "  ['" << dl->id() << "_" << cnt++ << "',['" << convertToXML(dl->name()) << "',[";
 
           if (dl->count()==1) // item with a unique name
           {
-            MemberDef  *md   = 0;
-            bool isMemberDef = d->definitionType()==Definition::TypeMember;
-            if (isMemberDef) md = (MemberDef*)d;
+            const MemberDef  *md = toMemberDef(d);
             QCString anchor = d->anchor();
 
             ti << "'" << externalRef("../",d->getReference(),TRUE)
-              << d->getOutputFileBase() << Doxygen::htmlFileExtension;
+              << addHtmlExtensionIfMissing(d->getOutputFileBase());
             if (!anchor.isEmpty())
             {
               ti << "#" << anchor;
@@ -1076,7 +1019,7 @@ void writeJavascriptSearchIndex()
             }
             else if (md)
             {
-              FileDef *fd = md->getBodyDef();
+              const FileDef *fd = md->getBodyDef();
               if (fd==0) fd = md->getFileDef();
               if (fd)
               {
@@ -1093,17 +1036,15 @@ void writeJavascriptSearchIndex()
           {
             QListIterator<Definition> di(*dl);
             bool overloadedFunction = FALSE;
-            Definition *prevScope = 0;
+            const Definition *prevScope = 0;
             int childCount=0;
             for (di.toFirst();(d=di.current());)
             {
               ++di;
-              Definition *scope     = d->getOuterScope();
-              Definition *next      = di.current();
-              Definition *nextScope = 0;
-              MemberDef  *md        = 0;
-              bool isMemberDef = d->definitionType()==Definition::TypeMember;
-              if (isMemberDef) md = (MemberDef*)d;
+              const Definition *scope     = d->getOuterScope();
+              const Definition *next      = di.current();
+              const Definition *nextScope = 0;
+              const MemberDef  *md        = toMemberDef(d);
               if (next) nextScope = next->getOuterScope();
               QCString anchor = d->anchor();
 
@@ -1112,7 +1053,7 @@ void writeJavascriptSearchIndex()
                 ti << "],[";
               }
               ti << "'" << externalRef("../",d->getReference(),TRUE)
-                << d->getOutputFileBase() << Doxygen::htmlFileExtension;
+                << addHtmlExtensionIfMissing(d->getOutputFileBase());
               if (!anchor.isEmpty())
               {
                 ti << "#" << anchor;
@@ -1147,20 +1088,20 @@ void writeJavascriptSearchIndex()
               QCString name;
               if (d->definitionType()==Definition::TypeClass)
               {
-                name = convertToXML(((ClassDef*)d)->displayName());
+                name = convertToXML((toClassDef(d))->displayName());
                 found = TRUE;
               }
               else if (d->definitionType()==Definition::TypeNamespace)
               {
-                name = convertToXML(((NamespaceDef*)d)->displayName());
+                name = convertToXML((toNamespaceDef(d))->displayName());
                 found = TRUE;
               }
               else if (scope==0 || scope==Doxygen::globalScope) // in global scope
               {
                 if (md)
                 {
-                  FileDef *fd = md->getBodyDef();
-                  if (fd==0) fd = md->getFileDef();
+                  const FileDef *fd = md->getBodyDef();
+                  if (fd==0) fd = md->resolveAlias()->getFileDef();
                   if (fd)
                   {
                     if (!prefix.isEmpty()) prefix+=":&#160;";
@@ -1169,7 +1110,7 @@ void writeJavascriptSearchIndex()
                   }
                 }
               }
-              else if (md && (md->getClassDef() || md->getNamespaceDef()))
+              else if (md && (md->resolveAlias()->getClassDef() || md->resolveAlias()->getNamespaceDef()))
                 // member in class or namespace scope
               {
                 SrcLangExt lang = md->getLanguage();
@@ -1279,13 +1220,14 @@ void writeJavascriptSearchIndex()
     ResourceMgr::instance().copyResource("search.js",searchDirName);
   }
   {
-    QFile f(searchDirName+"/nomatches.html");
+    QFile f(searchDirName+"/nomatches"+Doxygen::htmlFileExtension);
     if (f.open(IO_WriteOnly))
     {
       FTextStream t(&f);
       t << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" "
            "\"https://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">" << endl;
-      t << "<html><head><title></title>" << endl;
+      t << "<html xmlns=\"http://www.w3.org/1999/xhtml\">" << endl;
+      t << "<head><title></title>" << endl;
       t << "<meta http-equiv=\"Content-Type\" content=\"text/xhtml;charset=UTF-8\"/>" << endl;
       t << "<link rel=\"stylesheet\" type=\"text/css\" href=\"search.css\"/>" << endl;
       t << "<script type=\"text/javascript\" src=\"search.js\"></script>" << endl;
@@ -1319,7 +1261,7 @@ SearchIndexList::~SearchIndexList()
 {
 }
 
-void SearchIndexList::append(Definition *d)
+void SearchIndexList::append(const Definition *d)
 {
   QCString dispName = d->localName();
   SearchDefinitionList *l = find(dispName);
@@ -1327,11 +1269,11 @@ void SearchIndexList::append(Definition *d)
   {
     if (d->definitionType()==Definition::TypeGroup)
     {
-      dispName = ((GroupDef*)d)->groupTitle();
+      dispName = (toGroupDef(d))->groupTitle();
     }
     else if (d->definitionType()==Definition::TypePage)
     {
-      dispName = ((PageDef*)d)->title();
+      dispName = (toPageDef(d))->title();
     }
     l=new SearchDefinitionList(searchId(dispName),dispName);
     SDict< SearchDefinitionList >::append(dispName,l);
@@ -1375,7 +1317,7 @@ void initSearchIndexer()
   }
 }
 
-void finializeSearchIndexer()
+void finalizeSearchIndexer()
 {
   delete Doxygen::searchIndex;
 }
