@@ -854,16 +854,16 @@ class TextGeneratorSqlite3Impl : public TextGeneratorIntf
 };
 
 
-static bool bindTextParameter(SqlStmt &s,const char *name,const char *value, bool _static=TRUE)
+static bool bindTextParameter(SqlStmt &s,const char *name,const char *value, bool _static=FALSE)
 {
   int idx = sqlite3_bind_parameter_index(s.stmt, name);
   if (idx==0) {
-    msg("sqlite3_bind_parameter_index(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
+    err("sqlite3_bind_parameter_index(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
     return false;
   }
   int rv = sqlite3_bind_text(s.stmt, idx, value, -1, _static==TRUE?SQLITE_STATIC:SQLITE_TRANSIENT);
   if (rv!=SQLITE_OK) {
-    msg("sqlite3_bind_text(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
+    err("sqlite3_bind_text(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
     return false;
   }
   return true;
@@ -873,12 +873,12 @@ static bool bindIntParameter(SqlStmt &s,const char *name,int value)
 {
   int idx = sqlite3_bind_parameter_index(s.stmt, name);
   if (idx==0) {
-    msg("sqlite3_bind_parameter_index(%s)[%s] failed to find column: %s\n", name, s.query, sqlite3_errmsg(s.db));
+    err("sqlite3_bind_parameter_index(%s)[%s] failed to find column: %s\n", name, s.query, sqlite3_errmsg(s.db));
     return false;
   }
   int rv = sqlite3_bind_int(s.stmt, idx, value);
   if (rv!=SQLITE_OK) {
-    msg("sqlite3_bind_int(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
+    err("sqlite3_bind_int(%s)[%s] failed: %s\n", name, s.query, sqlite3_errmsg(s.db));
     return false;
   }
   return true;
@@ -909,11 +909,11 @@ static int insertPath(QCString name, bool local=TRUE, bool found=TRUE, int type=
 
   name = stripFromPath(name);
 
-  bindTextParameter(path_select,":name",name.data(),FALSE);
+  bindTextParameter(path_select,":name",name.data());
   rowid=step(path_select,TRUE,TRUE);
   if (rowid==0)
   {
-    bindTextParameter(path_insert,":name",name.data(),FALSE);
+    bindTextParameter(path_insert,":name",name.data());
     bindIntParameter(path_insert,":type",type);
     bindIntParameter(path_insert,":local",local?1:0);
     bindIntParameter(path_insert,":found",found?1:0);
@@ -925,9 +925,9 @@ static int insertPath(QCString name, bool local=TRUE, bool found=TRUE, int type=
 static void recordMetadata()
 {
   bindTextParameter(meta_insert,":doxygen_version",getFullVersion());
-  bindTextParameter(meta_insert,":schema_version","0.2.0"); //TODO: this should be a constant somewhere; not sure where
-  bindTextParameter(meta_insert,":generated_at",dateToString(TRUE), FALSE);
-  bindTextParameter(meta_insert,":generated_on",dateToString(FALSE), FALSE);
+  bindTextParameter(meta_insert,":schema_version","0.2.0",TRUE); //TODO: this should be a constant somewhere; not sure where
+  bindTextParameter(meta_insert,":generated_at",dateToString(TRUE));
+  bindTextParameter(meta_insert,":generated_on",dateToString(FALSE));
   bindTextParameter(meta_insert,":project_name",Config_getString(PROJECT_NAME));
   bindTextParameter(meta_insert,":project_number",Config_getString(PROJECT_NUMBER));
   bindTextParameter(meta_insert,":project_brief",Config_getString(PROJECT_BRIEF));
@@ -1155,7 +1155,7 @@ static int prepareStatement(sqlite3 *db, SqlStmt &s)
   rc = sqlite3_prepare_v2(db,s.query,-1,&s.stmt,0);
   if (rc!=SQLITE_OK)
   {
-    msg("prepare failed for %s\n%s\n", s.query, sqlite3_errmsg(db));
+    err("prepare failed for %s\n%s\n", s.query, sqlite3_errmsg(db));
     s.db = NULL;
     return -1;
   }
@@ -1226,7 +1226,7 @@ static int initializeTables(sqlite3* db)
     rc = sqlite3_exec(db, q, NULL, NULL, &errmsg);
     if (rc != SQLITE_OK)
     {
-      msg("failed to execute query: %s\n\t%s\n", q, errmsg);
+      err("failed to execute query: %s\n\t%s\n", q, errmsg);
       return -1;
     }
   }
@@ -1244,7 +1244,7 @@ static int initializeViews(sqlite3* db)
     rc = sqlite3_exec(db, q, NULL, NULL, &errmsg);
     if (rc != SQLITE_OK)
     {
-      msg("failed to execute query: %s\n\t%s\n", q, errmsg);
+      err("failed to execute query: %s\n\t%s\n", q, errmsg);
       return -1;
     }
   }
@@ -1264,13 +1264,9 @@ I think the hurdles are:
   inner_refid (unless I'm missing a method that would uniformly return
   the correct refid for all types).
 */
-static void writeInnerClasses(const ClassSDict *cl, struct Refid outer_refid)
+static void writeInnerClasses(const ClassLinkedRefMap &cl, struct Refid outer_refid)
 {
-  if (!cl) return;
-
-  ClassSDict::Iterator cli(*cl);
-  const ClassDef *cd;
-  for (cli.toFirst();(cd=cli.current());++cli)
+  for (const auto &cd : cl)
   {
     if (!cd->isHidden() && !cd->isAnonymous())
     {
@@ -1336,39 +1332,29 @@ static void writeInnerFiles(const FileList *fl, struct Refid outer_refid)
   }
 }
 
-static void writeInnerDirs(const DirList *dl, struct Refid outer_refid)
+static void writeInnerDirs(const DirList &dl, struct Refid outer_refid)
 {
-  if (dl)
+  for (const auto subdir : dl)
   {
-    QListIterator<DirDef> subdirs(*dl);
-    const DirDef *subdir;
-    for (subdirs.toFirst();(subdir=subdirs.current());++subdirs)
-    {
-      struct Refid inner_refid = insertRefid(subdir->getOutputFileBase());
+    struct Refid inner_refid = insertRefid(subdir->getOutputFileBase());
 
-      bindIntParameter(contains_insert,":inner_rowid", inner_refid.rowid);
-      bindIntParameter(contains_insert,":outer_rowid", outer_refid.rowid);
-      step(contains_insert);
-    }
+    bindIntParameter(contains_insert,":inner_rowid", inner_refid.rowid);
+    bindIntParameter(contains_insert,":outer_rowid", outer_refid.rowid);
+    step(contains_insert);
   }
 }
 
-static void writeInnerNamespaces(const NamespaceSDict *nl, struct Refid outer_refid)
+static void writeInnerNamespaces(const NamespaceLinkedRefMap &nl, struct Refid outer_refid)
 {
-  if (nl)
+  for (const auto &nd : nl)
   {
-    NamespaceSDict::Iterator nli(*nl);
-    const NamespaceDef *nd;
-    for (nli.toFirst();(nd=nli.current());++nli)
+    if (!nd->isHidden() && !nd->isAnonymous())
     {
-      if (!nd->isHidden() && !nd->isAnonymous())
-      {
-        struct Refid inner_refid = insertRefid(nd->getOutputFileBase());
+      struct Refid inner_refid = insertRefid(nd->getOutputFileBase());
 
-        bindIntParameter(contains_insert,":inner_rowid",inner_refid.rowid);
-        bindIntParameter(contains_insert,":outer_rowid",outer_refid.rowid);
-        step(contains_insert);
-      }
+      bindIntParameter(contains_insert,":inner_rowid",inner_refid.rowid);
+      bindIntParameter(contains_insert,":outer_rowid",outer_refid.rowid);
+      step(contains_insert);
     }
   }
 }
@@ -1426,10 +1412,14 @@ QCString getSQLDocBlock(const Definition *scope,
     fileName,
     lineNr,
     const_cast<Definition*>(scope),
-    dynamic_cast<const MemberDef*>(def),
+    toMemberDef(def),
     doc,
     FALSE,
-    FALSE
+    FALSE,
+    0,
+    FALSE,
+    FALSE,
+    Config_getBool(MARKDOWN_SUPPORT)
   );
   XMLCodeGenerator codeGen(t);
   // create a parse tree visitor for XML
@@ -1446,15 +1436,14 @@ static void getSQLDesc(SqlStmt &s,const char *col,const char *value,const Defini
   bindTextParameter(
     s,
     col,
-      getSQLDocBlock(
-        def->getOuterScope(),
-        def,
-        value,
-        def->docFile(),
-        def->docLine()
-      ),
-      FALSE
-    );
+    getSQLDocBlock(
+      def->getOuterScope(),
+      def,
+      value,
+      def->docFile(),
+      def->docLine()
+    )
+  );
 }
 ////////////////////////////////////////////
 
@@ -1621,32 +1610,22 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
     // + source references
     // The cross-references in initializers only work when both the src and dst
     // are defined.
-    MemberSDict *mdict = md->getReferencesMembers();
-    if (mdict!=0)
+    auto refList = md->getReferencesMembers();
+    for (const auto &rmd : refList)
     {
-      MemberSDict::IteratorDict mdi(*mdict);
-      const MemberDef *rmd;
-      for (mdi.toFirst();(rmd=mdi.current());++mdi)
-      {
-        insertMemberReference(md,rmd, "inline");
-      }
+      insertMemberReference(md,rmd, "inline");
     }
     // + source referenced by
-    mdict = md->getReferencedByMembers();
-    if (mdict!=0)
+    auto refByList = md->getReferencedByMembers();
+    for (const auto &rmd : refByList)
     {
-      MemberSDict::IteratorDict mdi(*mdict);
-      const MemberDef *rmd;
-      for (mdi.toFirst();(rmd=mdi.current());++mdi)
-      {
-        insertMemberReference(rmd,md, "inline");
-      }
+      insertMemberReference(rmd,md, "inline");
     }
     return;
   }
 
   bindIntParameter(memberdef_insert,":rowid", refid.rowid);
-  bindTextParameter(memberdef_insert,":kind",md->memberTypeName(),FALSE);
+  bindTextParameter(memberdef_insert,":kind",md->memberTypeName());
   bindIntParameter(memberdef_insert,":prot",md->protection());
 
   bindIntParameter(memberdef_insert,":static",md->isStatic());
@@ -1668,8 +1647,8 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   if (isFunc)
   {
     const ArgumentList &al = md->argumentList();
-    bindIntParameter(memberdef_insert,":const",al.constSpecifier);
-    bindIntParameter(memberdef_insert,":volatile",al.volatileSpecifier);
+    bindIntParameter(memberdef_insert,":const",al.constSpecifier());
+    bindIntParameter(memberdef_insert,":volatile",al.volatileSpecifier());
     bindIntParameter(memberdef_insert,":explicit",md->isExplicit());
     bindIntParameter(memberdef_insert,":inline",md->isInline());
     bindIntParameter(memberdef_insert,":final",md->isFinal());
@@ -1762,7 +1741,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
     linkifyText(TextGeneratorSqlite3Impl(l), def, md->getBodyDef(),md,typeStr);
     if (typeStr)
     {
-      bindTextParameter(memberdef_insert,":type",typeStr,FALSE);
+      bindTextParameter(memberdef_insert,":type",typeStr);
     }
 
     if (md->definition())
@@ -1807,7 +1786,7 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
 
   if ( md->getScopeString() )
   {
-    bindTextParameter(memberdef_insert,":scope",md->getScopeString(),FALSE);
+    bindTextParameter(memberdef_insert,":scope",md->getScopeString());
   }
 
   // +Brief, detailed and inbody description
@@ -1858,24 +1837,16 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   // + source references
   // The cross-references in initializers only work when both the src and dst
   // are defined.
-  MemberSDict *mdict = md->getReferencesMembers();
-  if (mdict!=0)
+  auto refList = md->getReferencesMembers();
+  for (const auto &refmd : refList)
   {
-    MemberSDict::IteratorDict mdi(*mdict);
-    for (mdi.toFirst();(rmd=mdi.current());++mdi)
-    {
-      insertMemberReference(md,rmd, "inline");
-    }
+    insertMemberReference(md,refmd, "inline");
   }
   // + source referenced by
-  mdict = md->getReferencedByMembers();
-  if (mdict!=0)
+  auto refByList = md->getReferencedByMembers();
+  for (const auto &refmd : refByList)
   {
-    MemberSDict::IteratorDict mdi(*mdict);
-    for (mdi.toFirst();(rmd=mdi.current());++mdi)
-    {
-      insertMemberReference(rmd,md, "inline");
-    }
+    insertMemberReference(refmd,md, "inline");
   }
 }
 
@@ -1904,20 +1875,13 @@ static void generateSqlite3Section( const Definition *d,
 
 static void associateAllClassMembers(const ClassDef *cd, struct Refid scope_refid)
 {
-  if (cd->memberNameInfoSDict())
+  for (auto &mni : cd->memberNameInfoLinkedMap())
   {
-    MemberNameInfoSDict::Iterator mnii(*cd->memberNameInfoSDict());
-    MemberNameInfo *mni;
-    for (mnii.toFirst();(mni=mnii.current());++mnii)
+    for (auto &mi : *mni)
     {
-      MemberNameInfoIterator mii(*mni);
-      MemberInfo *mi;
-      for (mii.toFirst();(mi=mii.current());++mii)
-      {
-        MemberDef *md = mi->memberDef;
-        QCString qrefid = md->getOutputFileBase() + "_1" + md->anchor();
-        associateMember(md, insertRefid(qrefid), scope_refid);
-      }
+      MemberDef *md = mi->memberDef();
+      QCString qrefid = md->getOutputFileBase() + "_1" + md->anchor();
+      associateMember(md, insertRefid(qrefid), scope_refid);
     }
   }
 }
@@ -1959,8 +1923,8 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   bindIntParameter(compounddef_insert,":rowid", refid.rowid);
 
   bindTextParameter(compounddef_insert,":name",cd->name());
-  bindTextParameter(compounddef_insert,":title",cd->title(), FALSE);
-  bindTextParameter(compounddef_insert,":kind",cd->compoundTypeString(),FALSE);
+  bindTextParameter(compounddef_insert,":title",cd->title());
+  bindTextParameter(compounddef_insert,":kind",cd->compoundTypeString());
   bindIntParameter(compounddef_insert,":prot",cd->protection());
 
   int file_id = insertPath(cd->getDefFileName());
@@ -2004,7 +1968,6 @@ static void generateSqlite3ForClass(const ClassDef *cd)
       DBG_CTX(("-----> ClassDef includeInfo for %s\n", nm.data()));
       DBG_CTX(("       local    : %d\n", ii->local));
       DBG_CTX(("       imported : %d\n", ii->imported));
-      DBG_CTX(("       indirect : %d\n", ii->indirect));
       DBG_CTX(("header: %s\n", ii->fileDef->absFilePath().data()));
       DBG_CTX(("       file_id  : %d\n", file_id));
       DBG_CTX(("       header_id: %d\n", header_id));
@@ -2022,41 +1985,31 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   step(compounddef_insert);
 
   // + list of direct super classes
-  if (cd->baseClasses())
+  for (const auto &bcd : cd->baseClasses())
   {
-    BaseClassListIterator bcli(*cd->baseClasses());
-    const BaseClassDef *bcd;
-    for (bcli.toFirst();(bcd=bcli.current());++bcli)
-    {
-      struct Refid base_refid = insertRefid(bcd->classDef->getOutputFileBase());
-      struct Refid derived_refid = insertRefid(cd->getOutputFileBase());
-      bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
-      bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
-      bindIntParameter(compoundref_insert,":prot",bcd->prot);
-      bindIntParameter(compoundref_insert,":virt",bcd->virt);
-      step(compoundref_insert);
-    }
+    struct Refid base_refid = insertRefid(bcd.classDef->getOutputFileBase());
+    struct Refid derived_refid = insertRefid(cd->getOutputFileBase());
+    bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
+    bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
+    bindIntParameter(compoundref_insert,":prot",bcd.prot);
+    bindIntParameter(compoundref_insert,":virt",bcd.virt);
+    step(compoundref_insert);
   }
 
   // + list of direct sub classes
-  if (cd->subClasses())
+  for (const auto &bcd : cd->subClasses())
   {
-    BaseClassListIterator bcli(*cd->subClasses());
-    const BaseClassDef *bcd;
-    for (bcli.toFirst();(bcd=bcli.current());++bcli)
-    {
-      struct Refid derived_refid = insertRefid(bcd->classDef->getOutputFileBase());
-      struct Refid base_refid = insertRefid(cd->getOutputFileBase());
-      bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
-      bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
-      bindIntParameter(compoundref_insert,":prot",bcd->prot);
-      bindIntParameter(compoundref_insert,":virt",bcd->virt);
-      step(compoundref_insert);
-    }
+    struct Refid derived_refid = insertRefid(bcd.classDef->getOutputFileBase());
+    struct Refid base_refid = insertRefid(cd->getOutputFileBase());
+    bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
+    bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
+    bindIntParameter(compoundref_insert,":prot",bcd.prot);
+    bindIntParameter(compoundref_insert,":virt",bcd.virt);
+    step(compoundref_insert);
   }
 
   // + list of inner classes
-  writeInnerClasses(cd->getClassSDict(),refid);
+  writeInnerClasses(cd->getClasses(),refid);
 
   // + template argument list(s)
   writeTemplateList(cd);
@@ -2106,8 +2059,8 @@ static void generateSqlite3ForNamespace(const NamespaceDef *nd)
   bindIntParameter(compounddef_insert,":rowid", refid.rowid);
 
   bindTextParameter(compounddef_insert,":name",nd->name());
-  bindTextParameter(compounddef_insert,":title",nd->title(), FALSE);
-  bindTextParameter(compounddef_insert,":kind","namespace",FALSE);
+  bindTextParameter(compounddef_insert,":title",nd->title());
+  bindTextParameter(compounddef_insert,":kind","namespace");
 
   int file_id = insertPath(nd->getDefFileName());
   bindIntParameter(compounddef_insert,":file_id",file_id);
@@ -2120,10 +2073,10 @@ static void generateSqlite3ForNamespace(const NamespaceDef *nd)
   step(compounddef_insert);
 
   // + contained class definitions
-  writeInnerClasses(nd->getClassSDict(),refid);
+  writeInnerClasses(nd->getClasses(),refid);
 
   // + contained namespace definitions
-  writeInnerNamespaces(nd->getNamespaceSDict(),refid);
+  writeInnerNamespaces(nd->getNamespaces(),refid);
 
   // + member groups
   if (nd->getMemberGroupSDict())
@@ -2172,9 +2125,9 @@ static void generateSqlite3ForFile(const FileDef *fd)
   if(!refid.created && compounddefExists(refid)){return;}
   bindIntParameter(compounddef_insert,":rowid", refid.rowid);
 
-  bindTextParameter(compounddef_insert,":name",fd->name(),FALSE);
-  bindTextParameter(compounddef_insert,":title",fd->title(),FALSE);
-  bindTextParameter(compounddef_insert,":kind","file",FALSE);
+  bindTextParameter(compounddef_insert,":name",fd->name());
+  bindTextParameter(compounddef_insert,":title",fd->title());
+  bindTextParameter(compounddef_insert,":kind","file");
 
   int file_id = insertPath(fd->getDefFileName());
   bindIntParameter(compounddef_insert,":file_id",file_id);
@@ -2220,7 +2173,6 @@ static void generateSqlite3ForFile(const FileDef *fd)
       DBG_CTX(("-----> FileDef includeInfo for %s\n", ii->includeName.data()));
       DBG_CTX(("       local:    %d\n", ii->local));
       DBG_CTX(("       imported: %d\n", ii->imported));
-      DBG_CTX(("       indirect: %d\n", ii->indirect));
       if(ii->fileDef)
       {
         DBG_CTX(("include: %s\n", ii->fileDef->absFilePath().data()));
@@ -2283,16 +2235,10 @@ static void generateSqlite3ForFile(const FileDef *fd)
   }
 
   // + contained class definitions
-  if (fd->getClassSDict())
-  {
-    writeInnerClasses(fd->getClassSDict(),refid);
-  }
+  writeInnerClasses(fd->getClasses(),refid);
 
   // + contained namespace definitions
-  if (fd->getNamespaceSDict())
-  {
-    writeInnerNamespaces(fd->getNamespaceSDict(),refid);
-  }
+  writeInnerNamespaces(fd->getNamespaces(),refid);
 
   // + member groups
   if (fd->getMemberGroupSDict())
@@ -2340,8 +2286,8 @@ static void generateSqlite3ForGroup(const GroupDef *gd)
   bindIntParameter(compounddef_insert,":rowid", refid.rowid);
 
   bindTextParameter(compounddef_insert,":name",gd->name());
-  bindTextParameter(compounddef_insert,":title",gd->groupTitle(), FALSE);
-  bindTextParameter(compounddef_insert,":kind","group",FALSE);
+  bindTextParameter(compounddef_insert,":title",gd->groupTitle());
+  bindTextParameter(compounddef_insert,":kind","group");
 
   int file_id = insertPath(gd->getDefFileName());
   bindIntParameter(compounddef_insert,":file_id",file_id);
@@ -2407,7 +2353,7 @@ static void generateSqlite3ForDir(const DirDef *dd)
   bindIntParameter(compounddef_insert,":rowid", refid.rowid);
 
   bindTextParameter(compounddef_insert,":name",dd->displayName());
-  bindTextParameter(compounddef_insert,":kind","dir",FALSE);
+  bindTextParameter(compounddef_insert,":kind","dir");
 
   int file_id = insertPath(dd->getDefFileName(),TRUE,TRUE,2);
   bindIntParameter(compounddef_insert,":file_id",file_id);
@@ -2431,7 +2377,7 @@ static void generateSqlite3ForDir(const DirDef *dd)
   step(compounddef_insert);
 
   // + files
-  writeInnerDirs(&dd->subDirs(),refid);
+  writeInnerDirs(dd->subDirs(),refid);
 
   // + files
   writeInnerFiles(dd->getFiles(),refid);
@@ -2490,9 +2436,9 @@ static void generateSqlite3ForPage(const PageDef *pd,bool isExample)
   }
 
   // + title
-  bindTextParameter(compounddef_insert,":title",title,FALSE);
+  bindTextParameter(compounddef_insert,":title",title);
 
-  bindTextParameter(compounddef_insert,":kind", isExample ? "example" : "page");
+  bindTextParameter(compounddef_insert,":kind", isExample ? "example" : "page",TRUE);
 
   int file_id = insertPath(pd->getDefFileName());
 
@@ -2514,29 +2460,36 @@ static void generateSqlite3ForPage(const PageDef *pd,bool isExample)
 static sqlite3* openDbConnection()
 {
 
-  QCString outputDirectory = Config_getString(OUTPUT_DIRECTORY);
+  QCString outputDirectory = Config_getString(SQLITE3_OUTPUT);
   QDir sqlite3Dir(outputDirectory);
   sqlite3 *db;
   int rc;
-  struct stat buf;
 
   rc = sqlite3_initialize();
   if (rc != SQLITE_OK)
   {
-    msg("sqlite3_initialize failed\n");
+    err("sqlite3_initialize failed\n");
     return NULL;
   }
 
+  QCString dbFileName = "doxygen_sqlite3.db";
+  QFileInfo fi(outputDirectory+"/"+dbFileName);
 
-  if (stat (outputDirectory+"/doxygen_sqlite3.db", &buf) == 0)
+  if (fi.exists())
   {
-    msg("doxygen_sqlite3.db already exists! aborting sqlite3 output generation!\n");
-    msg("If you wish to re-generate the database, remove or archive the existing copy first.\n");
-    return NULL;
+    if (Config_getBool(SQLITE3_RECREATE_DB))
+    {
+       QDir().remove(fi.absFilePath());
+    }
+    else
+    {
+      err("doxygen_sqlite3.db already exists! Rename, remove, or archive it to regenerate\n");
+      return NULL;
+    }
   }
 
   rc = sqlite3_open_v2(
-    outputDirectory+"/doxygen_sqlite3.db",
+    fi.absFilePath().utf8(),
     &db,
     SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE,
     0
@@ -2544,8 +2497,7 @@ static sqlite3* openDbConnection()
   if (rc != SQLITE_OK)
   {
     sqlite3_close(db);
-    msg("database open failed: %s\n", "doxygen_sqlite3.db");
-    return NULL;
+    err("Database open failed: %s\n", "doxygen_sqlite3.db");
   }
   return db;
 }
@@ -2588,21 +2540,17 @@ void generateSqlite3()
   recordMetadata();
 
   // + classes
-  ClassSDict::Iterator cli(*Doxygen::classSDict);
-  const ClassDef *cd;
-  for (cli.toFirst();(cd=cli.current());++cli)
+  for (const auto &cd : *Doxygen::classLinkedMap)
   {
     msg("Generating Sqlite3 output for class %s\n",cd->name().data());
-    generateSqlite3ForClass(cd);
+    generateSqlite3ForClass(cd.get());
   }
 
   // + namespaces
-  NamespaceSDict::Iterator nli(*Doxygen::namespaceSDict);
-  const NamespaceDef *nd;
-  for (nli.toFirst();(nd=nli.current());++nli)
+  for (const auto &nd : *Doxygen::namespaceLinkedMap)
   {
     msg("Generating Sqlite3 output for namespace %s\n",nd->name().data());
-    generateSqlite3ForNamespace(nd);
+    generateSqlite3ForNamespace(nd.get());
   }
 
   // + files
