@@ -1,13 +1,10 @@
 /******************************************************************************
  *
- * 
- *
- *
- * Copyright (C) 1997-2015 by Dimitri van Heesch.
+ * Copyright (C) 1997-2020 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -16,7 +13,7 @@
  *
  */
 
-#include <qfileinfo.h> 
+#include <qfileinfo.h>
 
 #include "xmldocvisitor.h"
 #include "docparser.h"
@@ -32,6 +29,7 @@
 #include "config.h"
 #include "htmlentity.h"
 #include "emoji.h"
+#include "filedef.h"
 
 static void visitCaption(XmlDocVisitor *parent, QList<DocNode> children)
 {
@@ -40,9 +38,9 @@ static void visitCaption(XmlDocVisitor *parent, QList<DocNode> children)
   for (cli.toFirst();(n=cli.current());++cli) n->accept(parent);
 }
 
-static void visitPreStart(FTextStream &t, const char *cmd, bool doCaption, 
-                          XmlDocVisitor *parent, QList<DocNode> children, 
-                          const QCString &name, bool writeType, DocImage::Type type, const QCString &width, 
+static void visitPreStart(FTextStream &t, const char *cmd, bool doCaption,
+                          XmlDocVisitor *parent, QList<DocNode> children,
+                          const QCString &name, bool writeType, DocImage::Type type, const QCString &width,
                           const QCString &height, bool inlineImage = FALSE)
 {
   t << "<" << cmd;
@@ -60,7 +58,7 @@ static void visitPreStart(FTextStream &t, const char *cmd, bool doCaption,
   }
   if (!name.isEmpty())
   {
-    t << " name=\"" << name << "\"";
+    t << " name=\"" << convertToXML(name, TRUE) << "\"";
   }
   if (!width.isEmpty())
   {
@@ -70,7 +68,10 @@ static void visitPreStart(FTextStream &t, const char *cmd, bool doCaption,
   {
     t << " height=\"" << convertToXML(height) << "\"";
   }
-  if (inlineImage) t << " inline=\"yes\">";
+  if (inlineImage)
+  {
+    t << " inline=\"yes\"";
+  }
   if (doCaption)
   {
     t << " caption=\"";
@@ -85,8 +86,8 @@ static void visitPostEnd(FTextStream &t, const char *cmd)
   t << "</" << cmd << ">" << endl;
 }
 
-XmlDocVisitor::XmlDocVisitor(FTextStream &t,CodeOutputInterface &ci) 
-  : DocVisitor(DocVisitor_XML), m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE) 
+XmlDocVisitor::XmlDocVisitor(FTextStream &t,CodeOutputInterface &ci)
+  : DocVisitor(DocVisitor_XML), m_t(t), m_ci(ci), m_insidePre(FALSE), m_hide(FALSE)
 {
 }
 
@@ -156,7 +157,7 @@ void XmlDocVisitor::visit(DocEmoji *s)
 void XmlDocVisitor::visit(DocURL *u)
 {
   if (m_hide) return;
-  m_t << "<ulink url=\""; 
+  m_t << "<ulink url=\"";
   if (u->isEmail()) m_t << "mailto:";
   filter(u->url());
   m_t << "\">";
@@ -184,12 +185,21 @@ void XmlDocVisitor::visit(DocStyleChange *s)
     case DocStyleChange::Bold:
       if (s->enable()) m_t << "<bold>";      else m_t << "</bold>";
       break;
+    case DocStyleChange::S:
+      if (s->enable()) m_t << "<s>";      else m_t << "</s>";
+      break;
     case DocStyleChange::Strike:
       if (s->enable()) m_t << "<strike>";      else m_t << "</strike>";
       break;
+    case DocStyleChange::Del:
+      if (s->enable()) m_t << "<del>";      else m_t << "</del>";
+      break;
     case DocStyleChange::Underline:
       if (s->enable()) m_t << "<underline>";      else m_t << "</underline>";
       break;
+    case DocStyleChange::Ins:
+      if (s->enable()) m_t << "<ins>";      else m_t << "</ins>";
+      break;
     case DocStyleChange::Italic:
       if (s->enable()) m_t << "<emphasis>";     else m_t << "</emphasis>";
       break;
@@ -209,12 +219,12 @@ void XmlDocVisitor::visit(DocStyleChange *s)
       if (s->enable()) m_t << "<small>";  else m_t << "</small>";
       break;
     case DocStyleChange::Preformatted:
-      if (s->enable()) 
+      if (s->enable())
       {
-        m_t << "<preformatted>";  
+        m_t << "<preformatted>";
         m_insidePre=TRUE;
       }
-      else 
+      else
       {
         m_t << "</preformatted>";
         m_insidePre=FALSE;
@@ -242,24 +252,48 @@ void XmlDocVisitor::visit(DocVerbatim *s)
           m_t << " filename=\"" << lang << "\">";
       else
           m_t << ">";
-      Doxygen::parserManager->getParser(lang)
-                            ->parseCode(m_ci,s->context(),s->text(),langExt,
-                                        s->isExample(),s->exampleFile());
-      m_t << "</programlisting>"; 
+      getCodeParser(lang).parseCode(m_ci,s->context(),s->text(),langExt,
+                                    s->isExample(),s->exampleFile());
+      m_t << "</programlisting>";
       break;
-    case DocVerbatim::Verbatim: 
+    case DocVerbatim::Verbatim:
       m_t << "<verbatim>";
       filter(s->text());
-      m_t << "</verbatim>"; 
+      m_t << "</verbatim>";
+      break;
+    case DocVerbatim::HtmlOnly:
+      if (s->isBlock())
+      {
+        m_t << "<htmlonly block=\"yes\">";
+      }
+      else
+      {
+        m_t << "<htmlonly>";
+      }
+      filter(s->text());
+      m_t << "</htmlonly>";
+      break;
+    case DocVerbatim::RtfOnly:
+      m_t << "<rtfonly>";
+      filter(s->text());
+      m_t << "</rtfonly>";
+      break;
+    case DocVerbatim::ManOnly:
+      m_t << "<manonly>";
+      filter(s->text());
+      m_t << "</manonly>";
+      break;
+    case DocVerbatim::LatexOnly:
+      m_t << "<latexonly>";
+      filter(s->text());
+      m_t << "</latexonly>";
       break;
-    case DocVerbatim::HtmlOnly: 
-    case DocVerbatim::RtfOnly: 
-    case DocVerbatim::ManOnly: 
-    case DocVerbatim::LatexOnly: 
     case DocVerbatim::DocbookOnly:
-      /* nothing */ 
+      m_t << "<docbookonly>";
+      filter(s->text());
+      m_t << "</docbookonly>";
       break;
-    case DocVerbatim::XmlOnly: 
+    case DocVerbatim::XmlOnly:
       m_t << s->text();
       break;
     case DocVerbatim::Dot:
@@ -293,30 +327,29 @@ void XmlDocVisitor::visit(DocInclude *inc)
   switch(inc->type())
   {
     case DocInclude::IncWithLines:
-      { 
+      {
          m_t << "<programlisting filename=\"" << inc->file() << "\">";
          QFileInfo cfi( inc->file() );
-         FileDef fd( cfi.dirPath().utf8(), cfi.fileName().utf8() );
-         Doxygen::parserManager->getParser(inc->extension())
-                               ->parseCode(m_ci,inc->context(),
+         FileDef *fd = createFileDef( cfi.dirPath().utf8(), cfi.fileName().utf8() );
+         getCodeParser(inc->extension()).parseCode(m_ci,inc->context(),
                                            inc->text(),
                                            langExt,
                                            inc->isExample(),
                                            inc->exampleFile(),
-                                           &fd,   // fileDef,
+                                           fd,   // fileDef,
                                            -1,    // start line
                                            -1,    // end line
                                            FALSE, // inline fragment
                                            0,     // memberDef
                                            TRUE   // show line numbers
 					   );
-         m_t << "</programlisting>"; 
+         delete fd;
+         m_t << "</programlisting>";
       }
-      break;    
-    case DocInclude::Include: 
+      break;
+    case DocInclude::Include:
       m_t << "<programlisting filename=\"" << inc->file() << "\">";
-      Doxygen::parserManager->getParser(inc->extension())
-                            ->parseCode(m_ci,inc->context(),
+      getCodeParser(inc->extension()).parseCode(m_ci,inc->context(),
                                         inc->text(),
                                         langExt,
                                         inc->isExample(),
@@ -328,11 +361,12 @@ void XmlDocVisitor::visit(DocInclude *inc)
                                         0,     // memberDef
                                         FALSE  // show line numbers
 				       );
-      m_t << "</programlisting>"; 
+      m_t << "</programlisting>";
       break;
-    case DocInclude::DontInclude: 
+    case DocInclude::DontInclude:
+    case DocInclude::DontIncWithLines:
       break;
-    case DocInclude::HtmlInclude: 
+    case DocInclude::HtmlInclude:
       if (inc->isBlock())
       {
         m_t << "<htmlonly block=\"yes\">";
@@ -349,47 +383,64 @@ void XmlDocVisitor::visit(DocInclude *inc)
       filter(inc->text());
       m_t << "</latexonly>";
       break;
-    case DocInclude::VerbInclude: 
+    case DocInclude::RtfInclude:
+      m_t << "<rtfonly>";
+      filter(inc->text());
+      m_t << "</rtfonly>";
+      break;
+    case DocInclude::ManInclude:
+      m_t << "<manonly>";
+      filter(inc->text());
+      m_t << "</manonly>";
+      break;
+    case DocInclude::XmlInclude:
+      filter(inc->text());
+      break;
+    case DocInclude::DocbookInclude:
+      m_t << "<docbookonly>";
+      filter(inc->text());
+      m_t << "</docbookonly>";
+      break;
+    case DocInclude::VerbInclude:
       m_t << "<verbatim>";
       filter(inc->text());
-      m_t << "</verbatim>"; 
+      m_t << "</verbatim>";
       break;
     case DocInclude::Snippet:
       m_t << "<programlisting filename=\"" << inc->file() << "\">";
-      Doxygen::parserManager->getParser(inc->extension())
-                            ->parseCode(m_ci,
+      getCodeParser(inc->extension()).parseCode(m_ci,
                                         inc->context(),
                                         extractBlock(inc->text(),inc->blockId()),
                                         langExt,
                                         inc->isExample(),
                                         inc->exampleFile()
                                        );
-      m_t << "</programlisting>"; 
+      m_t << "</programlisting>";
       break;
     case DocInclude::SnipWithLines:
       {
          m_t << "<programlisting filename=\"" << inc->file() << "\">";
          QFileInfo cfi( inc->file() );
-         FileDef fd( cfi.dirPath().utf8(), cfi.fileName().utf8() );
-         Doxygen::parserManager->getParser(inc->extension())
-                               ->parseCode(m_ci,
+         FileDef *fd = createFileDef( cfi.dirPath().utf8(), cfi.fileName().utf8() );
+         getCodeParser(inc->extension()).parseCode(m_ci,
                                            inc->context(),
                                            extractBlock(inc->text(),inc->blockId()),
                                            langExt,
                                            inc->isExample(),
-                                           inc->exampleFile(), 
-                                           &fd,
+                                           inc->exampleFile(),
+                                           fd,
                                            lineBlock(inc->text(),inc->blockId()),
                                            -1,    // endLine
                                            FALSE, // inlineFragment
                                            0,     // memberDef
                                            TRUE   // show line number
                                           );
-         m_t << "</programlisting>"; 
+         delete fd;
+         m_t << "</programlisting>";
       }
       break;
-    case DocInclude::SnippetDoc: 
-    case DocInclude::IncludeDoc: 
+    case DocInclude::SnippetDoc:
+    case DocInclude::IncludeDoc:
       err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
           "Please create a bug report\n",__FILE__);
       break;
@@ -398,9 +449,9 @@ void XmlDocVisitor::visit(DocInclude *inc)
 
 void XmlDocVisitor::visit(DocIncOperator *op)
 {
-  //printf("DocIncOperator: type=%d first=%d, last=%d text=`%s'\n",
+  //printf("DocIncOperator: type=%d first=%d, last=%d text='%s'\n",
   //    op->type(),op->isFirst(),op->isLast(),op->text().data());
-  if (op->isFirst()) 
+  if (op->isFirst())
   {
     if (!m_hide)
     {
@@ -409,24 +460,40 @@ void XmlDocVisitor::visit(DocIncOperator *op)
     pushEnabled();
     m_hide = TRUE;
   }
-  SrcLangExt langExt = getLanguageFromFileName(m_langExt);
-  if (op->type()!=DocIncOperator::Skip) 
+  QCString locLangExt = getFileNameExtension(op->includeFileName());
+  if (locLangExt.isEmpty()) locLangExt = m_langExt;
+  SrcLangExt langExt = getLanguageFromFileName(locLangExt);
+  if (op->type()!=DocIncOperator::Skip)
   {
     popEnabled();
-    if (!m_hide) 
+    if (!m_hide)
     {
-      Doxygen::parserManager->getParser(m_langExt)
-                            ->parseCode(m_ci,op->context(),
-                                        op->text(),langExt,op->isExample(),
-                                        op->exampleFile());
+      FileDef *fd = 0;
+      if (!op->includeFileName().isEmpty())
+      {
+        QFileInfo cfi( op->includeFileName() );
+        fd = createFileDef( cfi.dirPath().utf8(), cfi.fileName().utf8() );
+      }
+
+      getCodeParser(locLangExt).parseCode(m_ci,op->context(),
+                                          op->text(),langExt,op->isExample(),
+                                          op->exampleFile(),
+                                          fd,     // fileDef
+                                          op->line(),    // startLine
+                                          -1,    // endLine
+                                          FALSE, // inline fragment
+                                          0,     // memberDef
+                                          op->showLineNo()  // show line numbers
+                                         );
+      if (fd) delete fd;
     }
     pushEnabled();
     m_hide=TRUE;
   }
-  if (op->isLast())  
+  if (op->isLast())
   {
     popEnabled();
-    if (!m_hide) m_t << "</programlisting>"; 
+    if (!m_hide) m_t << "</programlisting>";
   }
   else
   {
@@ -506,13 +573,13 @@ void XmlDocVisitor::visitPre(DocAutoListItem *)
   m_t << "<listitem>";
 }
 
-void XmlDocVisitor::visitPost(DocAutoListItem *) 
+void XmlDocVisitor::visitPost(DocAutoListItem *)
 {
   if (m_hide) return;
   m_t << "</listitem>";
 }
 
-void XmlDocVisitor::visitPre(DocPara *) 
+void XmlDocVisitor::visitPre(DocPara *)
 {
   if (m_hide) return;
   m_t << "<para>";
@@ -540,21 +607,21 @@ void XmlDocVisitor::visitPre(DocSimpleSect *s)
   m_t << "<simplesect kind=\"";
   switch(s->type())
   {
-    case DocSimpleSect::See: 
+    case DocSimpleSect::See:
       m_t << "see"; break;
-    case DocSimpleSect::Return: 
+    case DocSimpleSect::Return:
       m_t << "return"; break;
-    case DocSimpleSect::Author: 
+    case DocSimpleSect::Author:
       m_t << "author"; break;
-    case DocSimpleSect::Authors: 
+    case DocSimpleSect::Authors:
       m_t << "authors"; break;
-    case DocSimpleSect::Version: 
+    case DocSimpleSect::Version:
       m_t << "version"; break;
-    case DocSimpleSect::Since: 
+    case DocSimpleSect::Since:
       m_t << "since"; break;
-    case DocSimpleSect::Date: 
+    case DocSimpleSect::Date:
       m_t << "date"; break;
-    case DocSimpleSect::Note: 
+    case DocSimpleSect::Note:
       m_t << "note"; break;
     case DocSimpleSect::Warning:
       m_t << "warning"; break;
@@ -570,9 +637,9 @@ void XmlDocVisitor::visitPre(DocSimpleSect *s)
       m_t << "remark"; break;
     case DocSimpleSect::Attention:
       m_t << "attention"; break;
-    case DocSimpleSect::User: 
+    case DocSimpleSect::User:
       m_t << "par"; break;
-    case DocSimpleSect::Rcs: 
+    case DocSimpleSect::Rcs:
       m_t << "rcs"; break;
     case DocSimpleSect::Unknown:  break;
   }
@@ -615,7 +682,7 @@ void XmlDocVisitor::visitPre(DocSimpleListItem *)
   m_t << "<listitem>";
 }
 
-void XmlDocVisitor::visitPost(DocSimpleListItem *) 
+void XmlDocVisitor::visitPost(DocSimpleListItem *)
 {
   if (m_hide) return;
   m_t << "</listitem>\n";
@@ -632,7 +699,7 @@ void XmlDocVisitor::visitPre(DocSection *s)
   m_t << "</title>" << endl;
 }
 
-void XmlDocVisitor::visitPost(DocSection *s) 
+void XmlDocVisitor::visitPost(DocSection *s)
 {
   m_t << "</sect" << s->level() << ">\n";
 }
@@ -640,18 +707,18 @@ void XmlDocVisitor::visitPost(DocSection *s)
 void XmlDocVisitor::visitPre(DocHtmlList *s)
 {
   if (m_hide) return;
-  if (s->type()==DocHtmlList::Ordered) 
-    m_t << "<orderedlist>\n"; 
-  else 
+  if (s->type()==DocHtmlList::Ordered)
+    m_t << "<orderedlist>\n";
+  else
     m_t << "<itemizedlist>\n";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlList *s) 
+void XmlDocVisitor::visitPost(DocHtmlList *s)
 {
   if (m_hide) return;
-  if (s->type()==DocHtmlList::Ordered) 
-    m_t << "</orderedlist>\n"; 
-  else 
+  if (s->type()==DocHtmlList::Ordered)
+    m_t << "</orderedlist>\n";
+  else
     m_t << "</itemizedlist>\n";
 }
 
@@ -661,7 +728,7 @@ void XmlDocVisitor::visitPre(DocHtmlListItem *)
   m_t << "<listitem>\n";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlListItem *) 
+void XmlDocVisitor::visitPost(DocHtmlListItem *)
 {
   if (m_hide) return;
   m_t << "</listitem>\n";
@@ -673,7 +740,7 @@ void XmlDocVisitor::visitPre(DocHtmlDescList *)
   m_t << "<variablelist>\n";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlDescList *) 
+void XmlDocVisitor::visitPost(DocHtmlDescList *)
 {
   if (m_hide) return;
   m_t << "</variablelist>\n";
@@ -685,7 +752,7 @@ void XmlDocVisitor::visitPre(DocHtmlDescTitle *)
   m_t << "<varlistentry><term>";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlDescTitle *) 
+void XmlDocVisitor::visitPost(DocHtmlDescTitle *)
 {
   if (m_hide) return;
   m_t << "</term></varlistentry>\n";
@@ -697,7 +764,7 @@ void XmlDocVisitor::visitPre(DocHtmlDescData *)
   m_t << "<listitem>";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlDescData *) 
+void XmlDocVisitor::visitPost(DocHtmlDescData *)
 {
   if (m_hide) return;
   m_t << "</listitem>\n";
@@ -706,11 +773,11 @@ void XmlDocVisitor::visitPost(DocHtmlDescData *)
 void XmlDocVisitor::visitPre(DocHtmlTable *t)
 {
   if (m_hide) return;
-  m_t << "<table rows=\"" << t->numRows() 
+  m_t << "<table rows=\"" << t->numRows()
       << "\" cols=\"" << t->numColumns() << "\">" ;
 }
 
-void XmlDocVisitor::visitPost(DocHtmlTable *) 
+void XmlDocVisitor::visitPost(DocHtmlTable *)
 {
   if (m_hide) return;
   m_t << "</table>\n";
@@ -722,7 +789,7 @@ void XmlDocVisitor::visitPre(DocHtmlRow *)
   m_t << "<row>\n";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlRow *) 
+void XmlDocVisitor::visitPost(DocHtmlRow *)
 {
   if (m_hide) return;
   m_t << "</row>\n";
@@ -731,13 +798,51 @@ void XmlDocVisitor::visitPost(DocHtmlRow *)
 void XmlDocVisitor::visitPre(DocHtmlCell *c)
 {
   if (m_hide) return;
-  if (c->isHeading()) m_t << "<entry thead=\"yes\">"; else m_t << "<entry thead=\"no\">";
+  if (c->isHeading()) m_t << "<entry thead=\"yes\""; else m_t << "<entry thead=\"no\"";
+  HtmlAttribListIterator li(c->attribs());
+  HtmlAttrib *opt;
+  for (li.toFirst();(opt=li.current());++li)
+  {
+    if (opt->name=="colspan" || opt->name=="rowspan")
+    {
+      m_t << " " << opt->name << "=\"" << opt->value.toInt() << "\"";
+    }
+    else if (opt->name=="align" &&
+             (opt->value=="right" || opt->value=="left" || opt->value=="center"))
+    {
+      m_t << " align=\"" << opt->value << "\"";
+    }
+    else if (opt->name=="class") // handle markdown generated attributes
+    {
+      if (opt->value.left(13)=="markdownTable") // handle markdown generated attributes
+      {
+        if (opt->value.right(5)=="Right")
+        {
+          m_t << " align='right'";
+        }
+        else if (opt->value.right(4)=="Left")
+        {
+          m_t << " align='left'";
+        }
+        else if (opt->value.right(6)=="Center")
+        {
+          m_t << " align='center'";
+        }
+        // skip 'markdownTable*' value ending with "None"
+      }
+      else if (!opt->value.isEmpty())
+      {
+        m_t << " class=\"" << convertToXML(opt->value) << "\"";
+      }
+    }
+  }
+  m_t << ">";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlCell *) 
+void XmlDocVisitor::visitPost(DocHtmlCell *)
 {
   if (m_hide) return;
-  m_t << "</entry>"; 
+  m_t << "</entry>";
 }
 
 void XmlDocVisitor::visitPre(DocHtmlCaption *)
@@ -746,7 +851,7 @@ void XmlDocVisitor::visitPre(DocHtmlCaption *)
   m_t << "<caption>";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlCaption *) 
+void XmlDocVisitor::visitPost(DocHtmlCaption *)
 {
   if (m_hide) return;
   m_t << "</caption>\n";
@@ -758,7 +863,7 @@ void XmlDocVisitor::visitPre(DocInternal *)
   m_t << "<internal>";
 }
 
-void XmlDocVisitor::visitPost(DocInternal *) 
+void XmlDocVisitor::visitPost(DocInternal *)
 {
   if (m_hide) return;
   m_t << "</internal>" << endl;
@@ -767,12 +872,10 @@ void XmlDocVisitor::visitPost(DocInternal *)
 void XmlDocVisitor::visitPre(DocHRef *href)
 {
   if (m_hide) return;
-  m_t << "<ulink url=\"";
-  filter(href->url());
-  m_t << "\">";
+  m_t << "<ulink url=\"" << convertToXML(href->url(), TRUE) << "\">";
 }
 
-void XmlDocVisitor::visitPost(DocHRef *) 
+void XmlDocVisitor::visitPost(DocHRef *)
 {
   if (m_hide) return;
   m_t << "</ulink>";
@@ -784,7 +887,7 @@ void XmlDocVisitor::visitPre(DocHtmlHeader *header)
   m_t << "<heading level=\"" << header->level() << "\">";
 }
 
-void XmlDocVisitor::visitPost(DocHtmlHeader *) 
+void XmlDocVisitor::visitPost(DocHtmlHeader *)
 {
   if (m_hide) return;
   m_t << "</heading>\n";
@@ -794,18 +897,22 @@ void XmlDocVisitor::visitPre(DocImage *img)
 {
   if (m_hide) return;
 
-  QCString baseName=img->name();
-  int i;
-  if ((i=baseName.findRev('/'))!=-1 || (i=baseName.findRev('\\'))!=-1)
+  QCString url = img->url();
+  QCString baseName;
+  if (url.isEmpty())
   {
-    baseName=baseName.right(baseName.length()-i-1);
+    baseName = img->relPath()+img->name();
+  }
+  else
+  {
+    baseName = correctURL(url,img->relPath());
   }
   visitPreStart(m_t, "image", FALSE, this, img->children(), baseName, TRUE, img->type(), img->width(), img->height(), img ->isInlineImage());
 
   // copy the image to the output dir
   FileDef *fd;
   bool ambig;
-  if ((fd=findFileDef(Doxygen::imageNameDict,img->name(),ambig)))
+  if (url.isEmpty() && (fd=findFileDef(Doxygen::imageNameLinkedMap,img->name(),ambig)))
   {
     QFile inImage(fd->absFilePath());
     QFile outImage(Config_getString(XML_OUTPUT)+"/"+baseName.data());
@@ -823,7 +930,7 @@ void XmlDocVisitor::visitPre(DocImage *img)
   }
 }
 
-void XmlDocVisitor::visitPost(DocImage *) 
+void XmlDocVisitor::visitPost(DocImage *)
 {
   if (m_hide) return;
   visitPostEnd(m_t, "image");
@@ -835,7 +942,7 @@ void XmlDocVisitor::visitPre(DocDotFile *df)
   visitPreStart(m_t, "dotfile", FALSE, this, df->children(), df->file(), FALSE, DocImage::Html, df->width(), df->height());
 }
 
-void XmlDocVisitor::visitPost(DocDotFile *) 
+void XmlDocVisitor::visitPost(DocDotFile *)
 {
   if (m_hide) return;
   visitPostEnd(m_t, "dotfile");
@@ -847,7 +954,7 @@ void XmlDocVisitor::visitPre(DocMscFile *df)
   visitPreStart(m_t, "mscfile", FALSE, this, df->children(), df->file(), FALSE, DocImage::Html, df->width(), df->height());
 }
 
-void XmlDocVisitor::visitPost(DocMscFile *) 
+void XmlDocVisitor::visitPost(DocMscFile *)
 {
   if (m_hide) return;
   visitPostEnd(m_t, "mscfile");
@@ -871,7 +978,7 @@ void XmlDocVisitor::visitPre(DocLink *lnk)
   startLink(lnk->ref(),lnk->file(),lnk->anchor());
 }
 
-void XmlDocVisitor::visitPost(DocLink *) 
+void XmlDocVisitor::visitPost(DocLink *)
 {
   if (m_hide) return;
   endLink();
@@ -880,14 +987,14 @@ void XmlDocVisitor::visitPost(DocLink *)
 void XmlDocVisitor::visitPre(DocRef *ref)
 {
   if (m_hide) return;
-  if (!ref->file().isEmpty()) 
+  if (!ref->file().isEmpty())
   {
     startLink(ref->ref(),ref->file(),ref->isSubPage() ? QCString() : ref->anchor());
   }
   if (!ref->hasLinkText()) filter(ref->targetTitle());
 }
 
-void XmlDocVisitor::visitPost(DocRef *ref) 
+void XmlDocVisitor::visitPost(DocRef *ref)
 {
   if (m_hide) return;
   if (!ref->file().isEmpty()) endLink();
@@ -900,7 +1007,7 @@ void XmlDocVisitor::visitPre(DocSecRefItem *ref)
   m_t << "<tocitem id=\"" << ref->file() << "_1" << ref->anchor() << "\">";
 }
 
-void XmlDocVisitor::visitPost(DocSecRefItem *) 
+void XmlDocVisitor::visitPost(DocSecRefItem *)
 {
   if (m_hide) return;
   m_t << "</tocitem>" << endl;
@@ -912,7 +1019,7 @@ void XmlDocVisitor::visitPre(DocSecRefList *)
   m_t << "<toclist>" << endl;
 }
 
-void XmlDocVisitor::visitPost(DocSecRefList *) 
+void XmlDocVisitor::visitPost(DocSecRefList *)
 {
   if (m_hide) return;
   m_t << "</toclist>" << endl;
@@ -924,7 +1031,7 @@ void XmlDocVisitor::visitPost(DocSecRefList *)
 //  m_t << "<language langid=\"" << l->id() << "\">";
 //}
 //
-//void XmlDocVisitor::visitPost(DocLanguage *) 
+//void XmlDocVisitor::visitPost(DocLanguage *)
 //{
 //  if (m_hide) return;
 //  m_t << "</language>" << endl;
@@ -936,13 +1043,13 @@ void XmlDocVisitor::visitPre(DocParamSect *s)
   m_t << "<parameterlist kind=\"";
   switch(s->type())
   {
-    case DocParamSect::Param: 
+    case DocParamSect::Param:
       m_t << "param"; break;
-    case DocParamSect::RetVal: 
+    case DocParamSect::RetVal:
       m_t << "retval"; break;
-    case DocParamSect::Exception: 
+    case DocParamSect::Exception:
       m_t << "exception"; break;
-    case DocParamSect::TemplateParam: 
+    case DocParamSect::TemplateParam:
       m_t << "templateparam"; break;
     default:
       ASSERT(0);
@@ -969,21 +1076,26 @@ void XmlDocVisitor::visitPre(DocParamList *pl)
   {
     if (pl->paramTypes().count()>0)
     {
-      QListIterator<DocNode> li(pl->paramTypes());
+      QListIterator<DocNode> li2(pl->paramTypes());
       DocNode *type;
-      for (li.toFirst();(type=li.current());++li)
+      m_t << "<parametertype>";
+      for (li2.toFirst();(type=li2.current());++li2)
       {
-        m_t << "<parametertype>";
         if (type->kind()==DocNode::Kind_Word)
         {
-          visit((DocWord*)type); 
+          visit((DocWord*)type);
         }
         else if (type->kind()==DocNode::Kind_LinkedWord)
         {
-          visit((DocLinkedWord*)type); 
+          visit((DocLinkedWord*)type);
+        }
+        else if (type->kind()==DocNode::Kind_Sep)
+        {
+          m_t << "</parametertype>" << endl;
+          m_t << "<parametertype>";
         }
-        m_t << "</parametertype>" << endl;
       }
+      m_t << "</parametertype>" << endl;
     }
     m_t << "<parametername";
     if (pl->direction()!=DocParamSect::Unspecified)
@@ -1006,11 +1118,11 @@ void XmlDocVisitor::visitPre(DocParamList *pl)
     m_t << ">";
     if (param->kind()==DocNode::Kind_Word)
     {
-      visit((DocWord*)param); 
+      visit((DocWord*)param);
     }
     else if (param->kind()==DocNode::Kind_LinkedWord)
     {
-      visit((DocLinkedWord*)param); 
+      visit((DocLinkedWord*)param);
     }
     m_t << "</parametername>" << endl;
   }
@@ -1052,25 +1164,13 @@ void XmlDocVisitor::visitPre(DocInternalRef *ref)
   startLink(0,ref->file(),ref->anchor());
 }
 
-void XmlDocVisitor::visitPost(DocInternalRef *) 
+void XmlDocVisitor::visitPost(DocInternalRef *)
 {
   if (m_hide) return;
   endLink();
   m_t << " ";
 }
 
-void XmlDocVisitor::visitPre(DocCopy *c)
-{
-  if (m_hide) return;
-  m_t << "<copydoc link=\"" << convertToXML(c->link()) << "\">";
-}
-
-void XmlDocVisitor::visitPost(DocCopy *)
-{
-  if (m_hide) return;
-  m_t << "</copydoc>" << endl;
-}
-
 void XmlDocVisitor::visitPre(DocText *)
 {
 }
@@ -1113,7 +1213,7 @@ void XmlDocVisitor::visitPost(DocParBlock *)
 
 
 void XmlDocVisitor::filter(const char *str)
-{ 
+{
   m_t << convertToXML(str);
 }
 
