@@ -154,7 +154,9 @@ static void writeXMLHeader(FTextStream &t)
   t << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>" << endl;;
   t << "<doxygen xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ";
   t << "xsi:noNamespaceSchemaLocation=\"compound.xsd\" ";
-  t << "version=\"" << getDoxygenVersion() << "\">" << endl;
+  t << "version=\"" << getDoxygenVersion() << "\" ";
+  t << "xml:lang=\"" << theTranslator->trISOLang() << "\"";
+  t << ">" << endl;
 }
 
 static void writeCombineScript()
@@ -178,7 +180,7 @@ static void writeCombineScript()
   "<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\">\n"
   "  <xsl:output method=\"xml\" version=\"1.0\" indent=\"no\" standalone=\"yes\" />\n"
   "  <xsl:template match=\"/\">\n"
-  "    <doxygen version=\"{doxygenindex/@version}\">\n"
+  "    <doxygen version=\"{doxygenindex/@version}\" xml:lang=\"{doxygenindex/@xml:lang}\">\n"
   "      <!-- Load all doxygen generated xml files -->\n"
   "      <xsl:for-each select=\"doxygenindex/compound\">\n"
   "        <xsl:copy-of select=\"document( concat( @refid, '.xml' ) )/doxygen/*\" />\n"
@@ -224,6 +226,7 @@ class TextGeneratorXMLImpl : public TextGeneratorIntf
     FTextStream &m_t;
 };
 
+//-------------------------------------------------------------------------------------------
 
 /** Generator for producing XML formatted source code. */
 void XMLCodeGenerator::codify(const char *text)
@@ -338,6 +341,18 @@ void XMLCodeGenerator::finish()
   if (m_insideCodeLine) endCodeLine();
 }
 
+void XMLCodeGenerator::startCodeFragment(const char *)
+{
+  m_t << "    <programlisting>" << endl;
+}
+
+void XMLCodeGenerator::endCodeFragment(const char *)
+{
+  m_t << "    </programlisting>" << endl;
+}
+
+//-------------------------------------------------------------------------------------------
+
 static void writeTemplateArgumentList(FTextStream &t,
                                       const ArgumentList &al,
                                       const Definition *scope,
@@ -401,7 +416,8 @@ static void writeXMLDocBlock(FTextStream &t,
   QCString stext = text.stripWhiteSpace();
   if (stext.isEmpty()) return;
   // convert the documentation string into an abstract syntax tree
-  DocNode *root = validatingParseDoc(fileName,lineNr,scope,md,text,FALSE,FALSE);
+  DocNode *root = validatingParseDoc(fileName,lineNr,scope,md,text,FALSE,FALSE,
+                                     0,FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
   // create a code generator
   XMLCodeGenerator *xmlCodeGen = new XMLCodeGenerator(t);
   // create a parse tree visitor for XML
@@ -417,11 +433,12 @@ static void writeXMLDocBlock(FTextStream &t,
 
 void writeXMLCodeBlock(FTextStream &t,FileDef *fd)
 {
-  CodeParserInterface &intf=Doxygen::parserManager->getCodeParser(fd->getDefFileExtension());
+  auto intf=Doxygen::parserManager->getCodeParser(fd->getDefFileExtension());
   SrcLangExt langExt = getLanguageFromFileName(fd->getDefFileExtension());
-  intf.resetCodeParserState();
+  intf->resetCodeParserState();
   XMLCodeGenerator *xmlGen = new XMLCodeGenerator(t);
-  intf.parseCode(*xmlGen,    // codeOutIntf
+  xmlGen->startCodeFragment("DoxyCode");
+  intf->parseCode(*xmlGen,    // codeOutIntf
                 0,           // scopeName
                 fileToString(fd->absFilePath(),Config_getBool(FILTER_SOURCE_FILES)),
                 langExt,     // lang
@@ -434,6 +451,7 @@ void writeXMLCodeBlock(FTextStream &t,FileDef *fd)
                 0,           // memberDef
                 TRUE         // showLineNumbers
                 );
+  xmlGen->endCodeFragment("DoxyCode");
   xmlGen->finish();
   delete xmlGen;
 }
@@ -464,6 +482,7 @@ static void writeMemberReference(FTextStream &t,const Definition *def,const Memb
 static void stripQualifiers(QCString &typeStr)
 {
   bool done=FALSE;
+  typeStr.stripPrefix("friend ");
   while (!done)
   {
     if (typeStr.stripPrefix("static "));
@@ -586,7 +605,7 @@ static void generateXMLForMember(const MemberDef *md,FTextStream &ti,FTextStream
   {
     const ArgumentList &al = md->argumentList();
     t << " const=\"";
-    if (al.constSpecifier)    t << "yes"; else t << "no";
+    if (al.constSpecifier())    t << "yes"; else t << "no";
     t << "\"";
 
     t << " explicit=\"";
@@ -597,10 +616,10 @@ static void generateXMLForMember(const MemberDef *md,FTextStream &ti,FTextStream
     if (md->isInline()) t << "yes"; else t << "no";
     t << "\"";
 
-    if (al.refQualifier!=RefQualifierNone)
+    if (al.refQualifier()!=RefQualifierNone)
     {
       t << " refqual=\"";
-      if (al.refQualifier==RefQualifierLValue) t << "lvalue"; else t << "rvalue";
+      if (al.refQualifier()==RefQualifierLValue) t << "lvalue"; else t << "rvalue";
       t << "\"";
     }
 
@@ -634,7 +653,7 @@ static void generateXMLForMember(const MemberDef *md,FTextStream &ti,FTextStream
       t << " noexcept=\"yes\"";
     }
 
-    if (al.volatileSpecifier)
+    if (al.volatileSpecifier())
     {
       t << " volatile=\"yes\"";
     }
@@ -831,7 +850,15 @@ static void generateXMLForMember(const MemberDef *md,FTextStream &ti,FTextStream
     }
   }
 
-  if (isFunc) //function
+  if (md->isFriendClass()) // for friend classes we show a link to the class as a "parameter"
+  {
+    t << "        <param>" << endl;
+    t << "          <type>";
+    linkifyText(TextGeneratorXMLImpl(t),def,md->getBodyDef(),md,md->name());
+    t << "</type>" << endl;
+    t << "        </param>" << endl;
+  }
+  else if (isFunc) //function
   {
     const ArgumentList &declAl = md->declArgumentList();
     const ArgumentList &defAl = md->argumentList();
@@ -1003,23 +1030,15 @@ static void generateXMLForMember(const MemberDef *md,FTextStream &ti,FTextStream
   }
 
   //printf("md->getReferencesMembers()=%p\n",md->getReferencesMembers());
-  MemberSDict *mdict = md->getReferencesMembers();
-  if (mdict)
+  auto refList = md->getReferencesMembers();
+  for (const auto &refmd : refList)
   {
-    MemberSDict::Iterator mdi(*mdict);
-    for (mdi.toFirst();(rmd=mdi.current());++mdi)
-    {
-      writeMemberReference(t,def,rmd,"references");
-    }
+    writeMemberReference(t,def,refmd,"references");
   }
-  mdict = md->getReferencedByMembers();
-  if (mdict)
+  auto refByList = md->getReferencedByMembers();
+  for (const auto &refmd : refByList)
   {
-    MemberSDict::Iterator mdi(*mdict);
-    for (mdi.toFirst();(rmd=mdi.current());++mdi)
-    {
-      writeMemberReference(t,def,rmd,"referencedby");
-    }
+    writeMemberReference(t,def,refmd,"referencedby");
   }
 
   t << "      </memberdef>" << endl;
@@ -1076,89 +1095,73 @@ static void generateXMLSection(const Definition *d,FTextStream &ti,FTextStream &
 static void writeListOfAllMembers(const ClassDef *cd,FTextStream &t)
 {
   t << "    <listofallmembers>" << endl;
-  if (cd->memberNameInfoSDict())
+  for (auto &mni : cd->memberNameInfoLinkedMap())
   {
-    MemberNameInfoSDict::Iterator mnii(*cd->memberNameInfoSDict());
-    MemberNameInfo *mni;
-    for (mnii.toFirst();(mni=mnii.current());++mnii)
+    for (auto &mi : *mni)
     {
-      MemberNameInfoIterator mii(*mni);
-      MemberInfo *mi;
-      for (mii.toFirst();(mi=mii.current());++mii)
+      const MemberDef *md=mi->memberDef();
+      if (!md->isAnonymous())
       {
-        const MemberDef *md=mi->memberDef;
-        if (!md->isAnonymous())
+        Protection prot = mi->prot();
+        Specifier virt=md->virtualness();
+        t << "      <member refid=\"" << memberOutputFileBase(md) << "_1" <<
+          md->anchor() << "\" prot=\"";
+        switch (prot)
         {
-          Protection prot = mi->prot;
-          Specifier virt=md->virtualness();
-          t << "      <member refid=\"" << memberOutputFileBase(md) << "_1" <<
-            md->anchor() << "\" prot=\"";
-          switch (prot)
-          {
-            case Public:    t << "public";    break;
-            case Protected: t << "protected"; break;
-            case Private:   t << "private";   break;
-            case Package:   t << "package";   break;
-          }
-          t << "\" virt=\"";
-          switch(virt)
-          {
-            case Normal:  t << "non-virtual";  break;
-            case Virtual: t << "virtual";      break;
-            case Pure:    t << "pure-virtual"; break;
-          }
-          t << "\"";
-          if (!mi->ambiguityResolutionScope.isEmpty())
-          {
-            t << " ambiguityscope=\"" << convertToXML(mi->ambiguityResolutionScope) << "\"";
-          }
-          t << "><scope>" << convertToXML(cd->name()) << "</scope><name>" <<
-            convertToXML(md->name()) << "</name></member>" << endl;
+          case Public:    t << "public";    break;
+          case Protected: t << "protected"; break;
+          case Private:   t << "private";   break;
+          case Package:   t << "package";   break;
         }
+        t << "\" virt=\"";
+        switch(virt)
+        {
+          case Normal:  t << "non-virtual";  break;
+          case Virtual: t << "virtual";      break;
+          case Pure:    t << "pure-virtual"; break;
+        }
+        t << "\"";
+        if (!mi->ambiguityResolutionScope().isEmpty())
+        {
+          t << " ambiguityscope=\"" << convertToXML(mi->ambiguityResolutionScope()) << "\"";
+        }
+        t << "><scope>" << convertToXML(cd->name()) << "</scope><name>" <<
+          convertToXML(md->name()) << "</name></member>" << endl;
       }
     }
   }
   t << "    </listofallmembers>" << endl;
 }
 
-static void writeInnerClasses(const ClassSDict *cl,FTextStream &t)
+static void writeInnerClasses(const ClassLinkedRefMap &cl,FTextStream &t)
 {
-  if (cl)
+  for (const auto &cd : cl)
   {
-    ClassSDict::Iterator cli(*cl);
-    const ClassDef *cd;
-    for (cli.toFirst();(cd=cli.current());++cli)
+    if (!cd->isHidden() && !cd->isAnonymous())
     {
-      if (!cd->isHidden() && !cd->isAnonymous())
+      t << "    <innerclass refid=\"" << classOutputFileBase(cd)
+        << "\" prot=\"";
+      switch(cd->protection())
       {
-        t << "    <innerclass refid=\"" << classOutputFileBase(cd)
-          << "\" prot=\"";
-        switch(cd->protection())
-        {
-           case Public:    t << "public";     break;
-           case Protected: t << "protected";  break;
-           case Private:   t << "private";    break;
-           case Package:   t << "package";    break;
-        }
-        t << "\">" << convertToXML(cd->name()) << "</innerclass>" << endl;
+         case Public:    t << "public";     break;
+         case Protected: t << "protected";  break;
+         case Private:   t << "private";    break;
+         case Package:   t << "package";    break;
       }
+      t << "\">" << convertToXML(cd->name()) << "</innerclass>" << endl;
     }
   }
 }
 
-static void writeInnerNamespaces(const NamespaceSDict *nl,FTextStream &t)
+static void writeInnerNamespaces(const NamespaceLinkedRefMap &nl,FTextStream &t)
 {
-  if (nl)
+  for (const auto &nd : nl)
   {
-    NamespaceSDict::Iterator nli(*nl);
-    const NamespaceDef *nd;
-    for (nli.toFirst();(nd=nli.current());++nli)
+    if (!nd->isHidden() && !nd->isAnonymous())
     {
-      if (!nd->isHidden() && !nd->isAnonymous())
-      {
-        t << "    <innernamespace refid=\"" << nd->getOutputFileBase()
-          << "\">" << convertToXML(nd->name()) << "</innernamespace>" << endl;
-      }
+      t << "    <innernamespace refid=\"" << nd->getOutputFileBase()
+        << "\"" << (nd->isInline() ? " inline=\"yes\"" : "")
+        << ">" << convertToXML(nd->name()) << "</innernamespace>" << endl;
     }
   }
 }
@@ -1214,9 +1217,7 @@ static void writeInnerDirs(const DirList *dl,FTextStream &t)
 {
   if (dl)
   {
-    QListIterator<DirDef> subdirs(*dl);
-    DirDef *subdir;
-    for (subdirs.toFirst();(subdir=subdirs.current());++subdirs)
+    for(const auto subdir : *dl)
     {
       t << "    <innerdir refid=\"" << subdir->getOutputFileBase()
         << "\">" << convertToXML(subdir->displayName()) << "</innerdir>" << endl;
@@ -1284,73 +1285,63 @@ static void generateXMLForClass(const ClassDef *cd,FTextStream &ti)
   t << "    <compoundname>";
   writeXMLString(t,cd->name());
   t << "</compoundname>" << endl;
-  if (cd->baseClasses())
+  for (const auto &bcd : cd->baseClasses())
   {
-    BaseClassListIterator bcli(*cd->baseClasses());
-    BaseClassDef *bcd;
-    for (bcli.toFirst();(bcd=bcli.current());++bcli)
+    t << "    <basecompoundref ";
+    if (bcd.classDef->isLinkable())
     {
-      t << "    <basecompoundref ";
-      if (bcd->classDef->isLinkable())
-      {
-        t << "refid=\"" << classOutputFileBase(bcd->classDef) << "\" ";
-      }
-      t << "prot=\"";
-      switch (bcd->prot)
-      {
-        case Public:    t << "public";    break;
-        case Protected: t << "protected"; break;
-        case Private:   t << "private";   break;
-        case Package: ASSERT(0); break;
-      }
-      t << "\" virt=\"";
-      switch(bcd->virt)
-      {
-        case Normal:  t << "non-virtual";  break;
-        case Virtual: t << "virtual";      break;
-        case Pure:    t <<"pure-virtual"; break;
-      }
-      t << "\">";
-      if (!bcd->templSpecifiers.isEmpty())
-      {
-        t << convertToXML(
-              insertTemplateSpecifierInScope(
-              bcd->classDef->name(),bcd->templSpecifiers)
-           );
-      }
-      else
-      {
-        t << convertToXML(bcd->classDef->displayName());
-      }
-      t  << "</basecompoundref>" << endl;
+      t << "refid=\"" << classOutputFileBase(bcd.classDef) << "\" ";
     }
+    t << "prot=\"";
+    switch (bcd.prot)
+    {
+      case Public:    t << "public";    break;
+      case Protected: t << "protected"; break;
+      case Private:   t << "private";   break;
+      case Package: ASSERT(0); break;
+    }
+    t << "\" virt=\"";
+    switch(bcd.virt)
+    {
+      case Normal:  t << "non-virtual";  break;
+      case Virtual: t << "virtual";      break;
+      case Pure:    t <<"pure-virtual"; break;
+    }
+    t << "\">";
+    if (!bcd.templSpecifiers.isEmpty())
+    {
+      t << convertToXML(
+          insertTemplateSpecifierInScope(
+            bcd.classDef->name(),bcd.templSpecifiers)
+          );
+    }
+    else
+    {
+      t << convertToXML(bcd.classDef->displayName());
+    }
+    t  << "</basecompoundref>" << endl;
   }
-  if (cd->subClasses())
+  for (const auto &bcd : cd->subClasses())
   {
-    BaseClassListIterator bcli(*cd->subClasses());
-    BaseClassDef *bcd;
-    for (bcli.toFirst();(bcd=bcli.current());++bcli)
+    t << "    <derivedcompoundref refid=\""
+      << classOutputFileBase(bcd.classDef)
+      << "\" prot=\"";
+    switch (bcd.prot)
     {
-      t << "    <derivedcompoundref refid=\""
-        << classOutputFileBase(bcd->classDef)
-        << "\" prot=\"";
-      switch (bcd->prot)
-      {
-        case Public:    t << "public";    break;
-        case Protected: t << "protected"; break;
-        case Private:   t << "private";   break;
-        case Package: ASSERT(0); break;
-      }
-      t << "\" virt=\"";
-      switch(bcd->virt)
-      {
-        case Normal:  t << "non-virtual";  break;
-        case Virtual: t << "virtual";      break;
-        case Pure:    t << "pure-virtual"; break;
-      }
-      t << "\">" << convertToXML(bcd->classDef->displayName())
-        << "</derivedcompoundref>" << endl;
+      case Public:    t << "public";    break;
+      case Protected: t << "protected"; break;
+      case Private:   t << "private";   break;
+      case Package: ASSERT(0); break;
     }
+    t << "\" virt=\"";
+    switch (bcd.virt)
+    {
+      case Normal:  t << "non-virtual";  break;
+      case Virtual: t << "virtual";      break;
+      case Pure:    t << "pure-virtual"; break;
+    }
+    t << "\">" << convertToXML(bcd.classDef->displayName())
+      << "</derivedcompoundref>" << endl;
   }
 
   IncludeInfo *ii=cd->includeInfo();
@@ -1371,7 +1362,7 @@ static void generateXMLForClass(const ClassDef *cd,FTextStream &ti)
     }
   }
 
-  writeInnerClasses(cd->getClassSDict(),t);
+  writeInnerClasses(cd->getClasses(),t);
 
   writeTemplateList(cd,t);
   if (cd->getMemberGroupSDict())
@@ -1467,14 +1458,16 @@ static void generateXMLForNamespace(const NamespaceDef *nd,FTextStream &ti)
 
   writeXMLHeader(t);
   t << "  <compounddef id=\"" << nd->getOutputFileBase()
-    << "\" kind=\"namespace\" language=\""
+    << "\" kind=\"namespace\" "
+    << (nd->isInline()?"inline=\"yes\" ":"")
+    << "language=\""
     << langToString(nd->getLanguage()) << "\">" << endl;
   t << "    <compoundname>";
   writeXMLString(t,nd->name());
   t << "</compoundname>" << endl;
 
-  writeInnerClasses(nd->getClassSDict(),t);
-  writeInnerNamespaces(nd->getNamespaceSDict(),t);
+  writeInnerClasses(nd->getClasses(),t);
+  writeInnerNamespaces(nd->getNamespaces(),t);
 
   if (nd->getMemberGroupSDict())
   {
@@ -1604,14 +1597,8 @@ static void generateXMLForFile(FileDef *fd,FTextStream &ti)
     t << "    </invincdepgraph>" << endl;
   }
 
-  if (fd->getClassSDict())
-  {
-    writeInnerClasses(fd->getClassSDict(),t);
-  }
-  if (fd->getNamespaceSDict())
-  {
-    writeInnerNamespaces(fd->getNamespaceSDict(),t);
-  }
+  writeInnerClasses(fd->getClasses(),t);
+  writeInnerNamespaces(fd->getNamespaces(),t);
 
   if (fd->getMemberGroupSDict())
   {
@@ -1642,9 +1629,7 @@ static void generateXMLForFile(FileDef *fd,FTextStream &ti)
   t << "    </detaileddescription>" << endl;
   if (Config_getBool(XML_PROGRAMLISTING))
   {
-    t << "    <programlisting>" << endl;
     writeXMLCodeBlock(t,fd);
-    t << "    </programlisting>" << endl;
   }
   t << "    <location file=\"" << convertToXML(stripFromPath(fd->getDefFileName())) << "\"/>" << endl;
   t << "  </compounddef>" << endl;
@@ -1916,6 +1901,7 @@ void generateXML()
   QDir xmlDir(outputDirectory);
   createSubDirs(xmlDir);
 
+  ResourceMgr::instance().copyResource("xml.xsd",outputDirectory);
   ResourceMgr::instance().copyResource("index.xsd",outputDirectory);
 
   QCString fileName=outputDirectory+"/compound.xsd";
@@ -1968,31 +1954,18 @@ void generateXML()
   t << "<?xml version='1.0' encoding='UTF-8' standalone='no'?>" << endl;;
   t << "<doxygenindex xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ";
   t << "xsi:noNamespaceSchemaLocation=\"index.xsd\" ";
-  t << "version=\"" << getDoxygenVersion() << "\">" << endl;
+  t << "version=\"" << getDoxygenVersion() << "\" ";
+  t << "xml:lang=\"" << theTranslator->trISOLang() << "\"";
+  t << ">" << endl;
 
+  for (const auto &cd : *Doxygen::classLinkedMap)
   {
-    ClassSDict::Iterator cli(*Doxygen::classSDict);
-    const ClassDef *cd;
-    for (cli.toFirst();(cd=cli.current());++cli)
-    {
-      generateXMLForClass(cd,t);
-    }
+    generateXMLForClass(cd.get(),t);
   }
-  //{
-  //  ClassSDict::Iterator cli(Doxygen::hiddenClasses);
-  //  ClassDef *cd;
-  //  for (cli.toFirst();(cd=cli.current());++cli)
-  //  {
-  //    msg("Generating XML output for class %s\n",cd->name().data());
-  //    generateXMLForClass(cd,t);
-  //  }
-  //}
-  NamespaceSDict::Iterator nli(*Doxygen::namespaceSDict);
-  const NamespaceDef *nd;
-  for (nli.toFirst();(nd=nli.current());++nli)
+  for (const auto &nd : *Doxygen::namespaceLinkedMap)
   {
     msg("Generating XML output for namespace %s\n",nd->name().data());
-    generateXMLForNamespace(nd,t);
+    generateXMLForNamespace(nd.get(),t);
   }
   for (const auto &fn : *Doxygen::inputNameLinkedMap)
   {
